@@ -8,7 +8,7 @@ use nextest_runner::{reporter::ReporterStderr, write_str::WriteStr};
 use owo_colors::{OwoColorize, Style, style};
 use std::{
     fmt,
-    io::{self, BufWriter, Stderr, Stdout, Write},
+    io::{self, BufWriter, IsTerminal, Stderr, Stdout, Write},
     marker::PhantomData,
 };
 use tracing::{
@@ -383,6 +383,16 @@ pub(crate) enum StdoutWriter<'a> {
     Test { buf: &'a mut String },
 }
 
+impl StdoutWriter<'_> {
+    pub(crate) fn is_terminal(&self) -> bool {
+        match self {
+            Self::Normal { buf, .. } => buf.get_ref().is_terminal(),
+            #[cfg(test)]
+            Self::Test { .. } => false,
+        }
+    }
+}
+
 impl WriteStr for StdoutWriter<'_> {
     fn write_str(&mut self, s: &str) -> io::Result<()> {
         match self {
