@@ -4,13 +4,14 @@
 use super::temp_project::TempProject;
 use camino::Utf8Path;
 use fixture_data::{
-    models::{CheckResult, RunProperty, TestSuiteFixtureProperty},
+    models::{CheckResult, RunProperties, TestCaseFixtureProperties, TestSuiteFixtureProperties},
     nextest_tests::EXPECTED_TEST_SUITES,
 };
 use iddqd::{IdOrdItem, IdOrdMap, id_upcast};
 use integration_tests::nextest_cli::{CargoNextestCli, cargo_bin};
 use nextest_metadata::{
-    BinaryListSummary, BuildPlatform, RustTestSuiteStatusSummary, TestListSummary,
+    BinaryListSummary, BuildPlatform, RustBinaryId, RustTestSuiteStatusSummary, TestCaseName,
+    TestListSummary,
 };
 use quick_junit::Report;
 use regex::Regex;
@@ -106,7 +107,7 @@ pub fn check_list_full_output(stdout: &[u8], platform: Option<BuildPlatform>) {
             test_suite.binary_id
         );
         for case in &test_suite.test_cases {
-            let e = entry.test_cases.get(case.name);
+            let e = entry.test_cases.get(&case.name);
             let e = match e {
                 Some(e) => e,
                 _ => panic!(
@@ -142,18 +143,104 @@ pub fn check_list_binaries_output(stdout: &[u8]) {
     );
 }
 
+/// Checks the output of `cargo nextest list --message-format oneline`.
+///
+/// The format is `binary_id test_name` per line.
+#[track_caller]
+pub fn check_list_oneline_output(stdout: &str) {
+    let test_suites = &*EXPECTED_TEST_SUITES;
+
+    // Build the set of expected test instances (binary_id, test_name).
+    let mut expected_tests: BTreeSet<(RustBinaryId, TestCaseName)> = BTreeSet::new();
+    for fixture in test_suites {
+        for test_case in &fixture.test_cases {
+            // Only include non-ignored tests (oneline without --verbose skips ignored tests).
+            if !test_case.status.is_ignored() {
+                expected_tests.insert((fixture.binary_id.clone(), test_case.name.clone()));
+            }
+        }
+    }
+
+    // Parse the actual output.
+    let mut actual_tests: BTreeSet<(RustBinaryId, TestCaseName)> = BTreeSet::new();
+    for line in stdout.lines() {
+        let parts: Vec<_> = line.splitn(2, ' ').collect();
+        assert_eq!(
+            parts.len(),
+            2,
+            "each line should be 'binary_id test_name', got: {line:?}"
+        );
+        let binary_id = parts[0];
+        let test_name = parts[1];
+        assert!(
+            !binary_id.is_empty(),
+            "binary_id should not be empty in line: {line:?}"
+        );
+        assert!(
+            !test_name.is_empty(),
+            "test_name should not be empty in line: {line:?}"
+        );
+        actual_tests.insert((RustBinaryId::new(binary_id), TestCaseName::new(test_name)));
+    }
+
+    // Compare expected vs actual.
+    let missing: Vec<_> = expected_tests.difference(&actual_tests).collect();
+    let extra: Vec<_> = actual_tests.difference(&expected_tests).collect();
+
+    assert!(
+        missing.is_empty() && extra.is_empty(),
+        "test list mismatch:\n  missing: {missing:?}\n  extra: {extra:?}"
+    );
+}
+
+/// Checks the output of `cargo nextest list --message-format oneline --list-type binaries-only`.
+///
+/// The format is `binary_id` per line.
+#[track_caller]
+pub fn check_list_oneline_binaries_output(stdout: &str) {
+    let test_suites = &*EXPECTED_TEST_SUITES;
+
+    // Build set of expected binary IDs.
+    let expected_binaries: BTreeSet<RustBinaryId> = test_suites
+        .iter()
+        .map(|fixture| fixture.binary_id.clone())
+        .collect();
+
+    // Parse the actual output.
+    let actual_binaries: BTreeSet<RustBinaryId> = stdout
+        .lines()
+        .map(|line| {
+            assert!(!line.is_empty(), "binary_id should not be empty");
+            assert!(
+                !line.contains(' '),
+                "binary_id should not contain spaces: {line:?}"
+            );
+            RustBinaryId::new(line)
+        })
+        .collect();
+
+    // Compare expected vs actual.
+    let missing: Vec<_> = expected_binaries.difference(&actual_binaries).collect();
+    let extra: Vec<_> = actual_binaries.difference(&expected_binaries).collect();
+
+    assert!(
+        missing.is_empty() && extra.is_empty(),
+        "binary list mismatch:\n  missing: {missing:?}\n  extra: {extra:?}"
+    );
+}
+
 /// Uniquely identifies a test case within the fixture data.
 #[derive(Clone, Debug, Eq, PartialEq, Hash, Ord, PartialOrd)]
 struct TestInstanceId {
     binary_id: String,
-    test_name: String,
+    test_name: TestCaseName,
 }
 
 impl TestInstanceId {
-    fn new(binary_id: &str, test_name: &str) -> Self {
+    fn new(binary_id: &str, test_name: &TestCaseName) -> Self {
         Self {
             binary_id: binary_id.to_owned(),
-            test_name: test_name.to_owned(),
+            test_name: test_name.clone(),
         }
     }
 
@@ -175,7 +262,7 @@ struct ExpectedTestResults {
 
 impl ExpectedTestResults {
     /// Builds expected test results by applying filters to fixture data based on properties.
-    fn new(properties: u64) -> Self {
+    fn new(properties: RunProperties) -> Self {
         let mut should_run = IdOrdMap::new();
         let mut should_not_run = BTreeSet::new();
         let mut summary = ExpectedSummary::default();
@@ -184,22 +271,29 @@ impl ExpectedTestResults {
             let binary_id = &fixture.binary_id;
 
             // Check if the entire test suite should be skipped.
-            let skip_suite = (fixture.has_property(TestSuiteFixtureProperty::NotInDefaultSet)
-                && properties & RunProperty::WithDefaultFilter as u64 != 0)
-                || (!fixture.has_property(TestSuiteFixtureProperty::MatchesCdylibExample)
-                    && properties & RunProperty::CdyLibExamplePackageFilter as u64 != 0);
+            let skip_suite = (fixture.has_property(TestSuiteFixtureProperties::NOT_IN_DEFAULT_SET)
+                && properties.contains(RunProperties::WITH_DEFAULT_FILTER))
+                || (!fixture.has_property(TestSuiteFixtureProperties::MATCHES_CDYLIB_EXAMPLE)
+                    && properties.contains(RunProperties::CDYLIB_EXAMPLE_PACKAGE_FILTER));
 
             if skip_suite {
                 // The entire suite should not appear in output.
                 for test in &fixture.test_cases {
-                    let identifier = TestInstanceId::new(binary_id.as_str(), test.name);
+                    let identifier = TestInstanceId::new(binary_id.as_str(), &test.name);
                     should_not_run.insert(identifier);
                 }
                 continue;
             }
 
             for test in &fixture.test_cases {
-                let id = TestInstanceId::new(binary_id.as_str(), test.name);
+                let id = TestInstanceId::new(binary_id.as_str(), &test.name);
+
+                if properties.contains(RunProperties::BENCHMARKS) {
+                    // We don't consider skipped tests while running benchmarks.
+                    if !test.has_property(TestCaseFixtureProperties::IS_BENCHMARK) {
+                        continue;
+                    }
+                }
 
                 // Determine if this specific test should be filtered out.
                 if test.should_skip(properties) {
@@ -279,6 +373,9 @@ impl ExpectedSummary {
             CheckResult::Abort => {
                 self.fail_count += 1;
             }
+            CheckResult::Timeout => {
+                self.fail_count += 1;
+            }
         }
     }
 }
@@ -286,17 +383,39 @@ impl ExpectedSummary {
 /// Test results parsed from actual test runner output.
 #[derive(Clone, Debug)]
 struct ActualTestResults {
-    /// Tests that appeared in output with their results.
+    /// Tests that appeared in output with all their attempts.
+    /// Each test may have multiple attempts if retries are enabled.
     tests: IdOrdMap<ActualOutcome>,
     /// The parsed summary line.
     summary: Option<ActualSummary>,
 }
 
+/// A single test attempt with its result.
+#[derive(Clone, Debug)]
+struct TestAttempt {
+    /// The attempt number (1-based). This is not currently used, but is tracked
+    /// internally.
+    #[expect(dead_code)]
+    attempt: u32,
+    result: CheckResult,
+}
+
 /// The actual outcome parsed from test output.
 #[derive(Clone, Debug)]
 struct ActualOutcome {
     id: TestInstanceId,
-    result: CheckResult,
+    /// All attempts for this test, in order of appearance.
+    attempts: Vec<TestAttempt>,
+}
+
+impl ActualOutcome {
+    /// Returns the final result (last attempt).
+    fn final_result(&self) -> CheckResult {
+        self.attempts
+            .last()
+            .expect("at least one attempt should exist")
+            .result
+    }
 }
 
 impl IdOrdItem for ActualOutcome {
@@ -318,109 +437,143 @@ struct ActualSummary {
     skip_count: usize,
 }
 
-fn debug_run_properties(properties: u64) -> String {
-    let mut ret = String::new();
-    if properties & RunProperty::Relocated as u64 != 0 {
-        ret.push_str("relocated ");
-    }
-    if properties & RunProperty::WithDefaultFilter as u64 != 0 {
-        ret.push_str("with-default-filter ");
-    }
-    if properties & RunProperty::WithSkipCdylibFilter as u64 != 0 {
-        ret.push_str("with-skip-cdylib-filter ");
-    }
-    if properties & RunProperty::WithMultiplyTwoExactFilter as u64 != 0 {
-        ret.push_str("with-exact-filter ");
-    }
-    if properties & RunProperty::CdyLibExamplePackageFilter as u64 != 0 {
-        ret.push_str("with-dylib-package-filter ");
-    }
-    if properties & RunProperty::SkipSummaryCheck as u64 != 0 {
-        ret.push_str("with-skip-summary-check ");
-    }
-    if properties & RunProperty::ExpectNoBinaries as u64 != 0 {
-        ret.push_str("with-expect-no-binaries ");
-    }
-    ret
+fn debug_run_properties(properties: RunProperties) -> String {
+    format!("{properties:?}")
 }
 
 // Regex patterns for parsing test result lines from nextest output.
-// Format: (STATUS) [duration] (attempt info) binary_id test_name
+// Format: (TRY N )?(STATUS) [duration] (count/total or progress) binary_id test_name
 // Example: "        PASS [   0.004s] (  1/249) nextest-runner cargo_config::test_..."
-static PASS_RE: LazyLock<Regex> =
-    LazyLock::new(|| Regex::new(r"^\s+PASS \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap());
-static LEAK_RE: LazyLock<Regex> =
-    LazyLock::new(|| Regex::new(r"^\s+LEAK \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap());
-static LEAK_FAIL_RE: LazyLock<Regex> =
-    LazyLock::new(|| Regex::new(r"^\s+LEAK-FAIL \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap());
-static FAIL_RE: LazyLock<Regex> =
-    LazyLock::new(|| Regex::new(r"^\s+FAIL \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap());
-static FAIL_LEAK_RE: LazyLock<Regex> =
-    LazyLock::new(|| Regex::new(r"^\s+FAIL \+ LEAK \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap());
+// For flaky tests that eventually pass, the format includes "TRY N " prefix:
+// Example: "  TRY 3 PASS [   1.003s] (1/1) nextest-tests::basic test_flaky..."
+//
+// We capture ALL result lines (including intermediate TRY N lines with progress like "(─────)")
+// to track all attempts. The attempt number is captured in group 1 (if present).
+// Groups: 1=attempt (optional), 2=binary_id, 3=test_name
+//
+// NOTE: We use \s* (zero or more whitespace) instead of \s+ because some lines may have
+// varying amounts of leading whitespace depending on the test status and retry attempt.
+static PASS_RE: LazyLock<Regex> = LazyLock::new(|| {
+    Regex::new(r"^\s*(?:TRY (\d+) )?PASS \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap()
+});
+static LEAK_RE: LazyLock<Regex> = LazyLock::new(|| {
+    Regex::new(r"^\s*(?:TRY (\d+) )?LEAK \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap()
+});
+static LEAK_FAIL_RE: LazyLock<Regex> = LazyLock::new(|| {
+    Regex::new(r"^\s*(?:TRY (\d+) )?(?:LEAK-FAIL|LKFAIL) \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)")
+        .unwrap()
+});
+static FAIL_RE: LazyLock<Regex> = LazyLock::new(|| {
+    Regex::new(r"^\s*(?:TRY (\d+) )?FAIL \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap()
+});
+static FAIL_LEAK_RE: LazyLock<Regex> = LazyLock::new(|| {
+    // Match both "FAIL + LEAK" (first attempt) and "FL+LK" (retry attempts).
+    Regex::new(r"^\s*(?:TRY (\d+) )?(?:FAIL \+ LEAK|FL\+LK) \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)")
+        .unwrap()
+});
 static ABORT_RE: LazyLock<Regex> = LazyLock::new(|| {
-    Regex::new(r"^\s+(?:ABORT|SIGSEGV|SIGABRT) \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap()
+    Regex::new(
+        r"^\s*(?:TRY (\d+) )?(?:ABORT|ABRT|SIGSEGV|SIGABRT) \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)",
+    )
+    .unwrap()
+});
+static TIMEOUT_RE: LazyLock<Regex> = LazyLock::new(|| {
+    Regex::new(r"^\s*(?:TRY (\d+) )?TIMEOUT \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap()
+});
+// FLAKY is shown in the summary section for tests that eventually passed.
+// Format: "FLAKY 4/5 [duration] (count/total) binary_id test_name"
+static FLAKY_RE: LazyLock<Regex> = LazyLock::new(|| {
+    Regex::new(r"^\s*FLAKY (\d+)/(\d+) \[[^\]]+\] \([^\)]+\) +(.+?) +(.+)").unwrap()
 });
 static SUMMARY_RE: LazyLock<Regex> = LazyLock::new(|| {
-    Regex::new(r"Summary \[.*\] +(\d+) tests? run: (\d+) passed(?: \((\d+) leaky\))?,?(?: (\d+) failed(?: \((\d+) due to being leaky\))?,?)? (\d+) skipped").unwrap()
+    // Note: "failed" can also be "timed out" for timeout failures.
+    Regex::new(r"Summary \[.*\] +(\d+) (?:tests?|benchmarks?) run: (\d+) passed(?: \((\d+) leaky\))?,?(?: (\d+) (?:failed|timed out)(?: \((\d+) due to being leaky\))?,?)? (\d+) skipped").unwrap()
 });
 
 impl ActualTestResults {
     /// Parses test results from nextest output.
+    ///
+    /// This function tracks ALL attempts for each test, not just the final result.
+    /// With retries enabled, a test may appear multiple times in the output with
+    /// different results (e.g., TRY 1 FAIL, TRY 2 FAIL, TRY 3 PASS).
     fn parse(output: &str) -> Self {
         let mut tests = IdOrdMap::new();
         let mut summary = None;
 
+        /// Helper to add an attempt to the test results.
+        fn add_attempt(
+            tests: &mut IdOrdMap<ActualOutcome>,
+            caps: &regex::Captures,
+            result: CheckResult,
+        ) {
+            // Groups: 1=attempt (optional), 2=binary_id, 3=test_name
+            let attempt = match caps.get(1) {
+                Some(m) => m.as_str().parse::<u32>().expect("parsed attempt number"),
+                None => 1,
+            };
+            let test_name = TestCaseName::new(&caps[3]);
+            let id = TestInstanceId::new(&caps[2], &test_name);
+            let attempt_record = TestAttempt { attempt, result };
+
+            match tests.entry(&id) {
+                iddqd::id_ord_map::Entry::Occupied(mut entry) => {
+                    entry.get_mut().attempts.push(attempt_record);
+                }
+                iddqd::id_ord_map::Entry::Vacant(entry) => {
+                    entry.insert(ActualOutcome {
+                        id,
+                        attempts: vec![attempt_record],
+                    });
+                }
+            }
+        }
+
         // Parse each line for test results. Check more specific patterns first
         // (e.g., "FAIL + LEAK" before "FAIL" or "LEAK").
         for line in output.lines() {
+            // Track all attempts for each test to analyze retry behavior.
             if let Some(caps) = FAIL_LEAK_RE.captures(line) {
-                let id = TestInstanceId::new(&caps[1], &caps[2]);
-                tests
-                    .insert_unique(ActualOutcome {
-                        id,
-                        result: CheckResult::FailLeak,
-                    })
-                    .unwrap();
+                add_attempt(&mut tests, &caps, CheckResult::FailLeak);
             } else if let Some(caps) = LEAK_FAIL_RE.captures(line) {
-                let id = TestInstanceId::new(&caps[1], &caps[2]);
-                tests
-                    .insert_unique(ActualOutcome {
-                        id,
-                        result: CheckResult::LeakFail,
-                    })
-                    .unwrap();
+                add_attempt(&mut tests, &caps, CheckResult::LeakFail);
             } else if let Some(caps) = ABORT_RE.captures(line) {
-                let id = TestInstanceId::new(&caps[1], &caps[2]);
-                tests
-                    .insert_unique(ActualOutcome {
-                        id,
-                        result: CheckResult::Abort,
-                    })
-                    .unwrap();
+                add_attempt(&mut tests, &caps, CheckResult::Abort);
+            } else if let Some(caps) = TIMEOUT_RE.captures(line) {
+                add_attempt(&mut tests, &caps, CheckResult::Timeout);
             } else if let Some(caps) = LEAK_RE.captures(line) {
-                let id = TestInstanceId::new(&caps[1], &caps[2]);
-                tests
-                    .insert_unique(ActualOutcome {
-                        id,
-                        result: CheckResult::Leak,
-                    })
-                    .unwrap();
+                add_attempt(&mut tests, &caps, CheckResult::Leak);
             } else if let Some(caps) = FAIL_RE.captures(line) {
-                let id = TestInstanceId::new(&caps[1], &caps[2]);
-                tests
-                    .insert_unique(ActualOutcome {
-                        id,
-                        result: CheckResult::Fail,
-                    })
-                    .unwrap();
+                add_attempt(&mut tests, &caps, CheckResult::Fail);
             } else if let Some(caps) = PASS_RE.captures(line) {
-                let id = TestInstanceId::new(&caps[1], &caps[2]);
-                tests
-                    .insert_unique(ActualOutcome {
-                        id,
-                        result: CheckResult::Pass,
-                    })
-                    .unwrap();
+                add_attempt(&mut tests, &caps, CheckResult::Pass);
+            } else if let Some(caps) = FLAKY_RE.captures(line) {
+                // FLAKY appears in summary section for tests that eventually passed.
+                // It's in a different format: "FLAKY 4/5 [duration] (count/total) binary_id test_name"
+                // Groups: 1=pass_attempt, 2=total_attempts, 3=binary_id, 4=test_name
+                // We record this as a PASS since the test eventually passed.
+                let test_name = TestCaseName::new(&caps[4]);
+                let id = TestInstanceId::new(&caps[3], &test_name);
+                let attempt_record = TestAttempt {
+                    attempt: caps[1].parse().expect("parsed attempt number"),
+                    result: CheckResult::Pass,
+                };
+
+                match tests.entry(&id) {
+                    iddqd::id_ord_map::Entry::Occupied(mut entry) => {
+                        // FLAKY line appears after individual attempts, so it may
+                        // duplicate the final PASS. Only add if not already a PASS.
+                        let attempts = &mut entry.get_mut().attempts;
+                        if attempts.last().map(|a| a.result) != Some(CheckResult::Pass) {
+                            attempts.push(attempt_record);
+                        }
+                    }
+                    iddqd::id_ord_map::Entry::Vacant(entry) => {
+                        entry.insert(ActualOutcome {
+                            id,
+                            attempts: vec![attempt_record],
+                        });
+                    }
+                }
             } else if let Some(caps) = SUMMARY_RE.captures(line) {
                 let run_count = caps[1].parse().unwrap();
                 let pass_count = caps[2].parse().unwrap();
@@ -470,15 +623,18 @@ fn verify_expected_in_actual(
 
         match actual_outcome {
             Some(actual) => {
-                // Test is present, verify result matches.
+                // Test is present, verify the final result matches.
+                // With retries, a test may have multiple attempts - we check the last one.
+                let actual_result = actual.final_result();
                 assert_eq!(
                     expected_outcome.result,
-                    actual.result,
-                    "{}: expected result {:?} but got {:?}\n\n\
+                    actual_result,
+                    "{}: expected result {:?} but got {:?} (attempts: {:?})\n\n\
                      --- output ---\n{}\n--- end output ---",
                     expected_outcome.id.full_name(),
                     expected_outcome.result,
-                    actual.result,
+                    actual_result,
+                    actual.attempts,
                     output
                 );
             }
@@ -553,10 +709,10 @@ fn verify_summary(
     expected_summary: &ExpectedSummary,
     actual_summary: Option<&ActualSummary>,
     output: &str,
-    properties: u64,
+    properties: RunProperties,
 ) {
     // Skip the summary check if requested.
-    if properties & RunProperty::SkipSummaryCheck as u64 != 0 {
+    if properties.contains(RunProperties::SKIP_SUMMARY_CHECK) {
         return;
     }
 
@@ -618,12 +774,16 @@ fn verify_summary(
 }
 
 #[track_caller]
-pub fn check_run_output_with_junit(stderr: &[u8], junit_path: &Utf8Path, properties: u64) {
+pub fn check_run_output_with_junit(
+    stderr: &[u8],
+    junit_path: &Utf8Path,
+    properties: RunProperties,
+) {
     check_run_output_impl(stderr, Some(junit_path), properties);
 }
 
 #[track_caller]
-fn check_run_output_impl(stderr: &[u8], junit_path: Option<&Utf8Path>, properties: u64) {
+fn check_run_output_impl(stderr: &[u8], junit_path: Option<&Utf8Path>, properties: RunProperties) {
     let output = String::from_utf8(stderr.to_vec()).unwrap();
 
     println!("{output}");
@@ -632,6 +792,9 @@ fn check_run_output_impl(stderr: &[u8], junit_path: Option<&Utf8Path>, propertie
     let expected = ExpectedTestResults::new(properties);
     let actual = ActualTestResults::parse(&output);
 
+    eprintln!("expected: {expected:?}");
+    eprintln!("actual: {actual:?}");
+
     // Check that all expected tests appear (or don't appear) as required.
     verify_expected_in_actual(&expected, &actual, &output);
 
@@ -695,8 +858,8 @@ impl ActualJunitResults {
             }
 
             for test_case in &test_suite.test_cases {
-                let test_name = test_case.name.as_str();
-                let id = TestInstanceId::new(binary_id, test_name);
+                let test_name = TestCaseName::new(test_case.name.as_str());
+                let id = TestInstanceId::new(binary_id, &test_name);
 
                 let non_success = match &test_case.status {
                     quick_junit::TestCaseStatus::Success { .. } => None,
@@ -728,7 +891,7 @@ impl ActualJunitResults {
 
 /// Verifies that JUnit output matches expected test results.
 #[track_caller]
-fn verify_junit(expected: &ExpectedTestResults, junit_path: &Utf8Path, properties: u64) {
+fn verify_junit(expected: &ExpectedTestResults, junit_path: &Utf8Path, properties: RunProperties) {
     let actual = ActualJunitResults::parse(junit_path);
 
     verify_expected_in_junit(expected, &actual, junit_path, properties);
@@ -736,6 +899,12 @@ fn verify_junit(expected: &ExpectedTestResults, junit_path: &Utf8Path, propertie
     verify_junit_in_expected(&actual, expected, junit_path, properties);
 }
 
+// Expected JUnit type strings. These match the strings produced by nextest in
+// junit.rs. The Rust test harness always uses exit code 101 for test failures.
+const JUNIT_FAIL: &str = "test failure with exit code 101";
+const JUNIT_FAIL_LEAK: &str = "test failure with exit code 101, and leaked handles";
+const JUNIT_ABORT: &str = "test abort";
+
 /// Verifies that all expected tests appear in the JUnit output with the correct
 /// status.
 #[track_caller]
@@ -743,7 +912,7 @@ fn verify_expected_in_junit(
     expected: &ExpectedTestResults,
     actual: &ActualJunitResults,
     junit_path: &Utf8Path,
-    properties: u64,
+    properties: RunProperties,
 ) {
     for expected_outcome in &expected.should_run {
         let actual_outcome = actual.tests.get(&expected_outcome.id);
@@ -751,33 +920,44 @@ fn verify_expected_in_junit(
         match actual_outcome {
             Some(actual) => {
                 // Verify the JUnit status matches our expected CheckResult.
-                let expected_junit = match expected_outcome.result {
-                    CheckResult::Pass | CheckResult::Leak => None,
-                    CheckResult::LeakFail => Some((
-                        quick_junit::NonSuccessKind::Error,
-                        "test exited with code 0, but leaked handles so was marked failed",
-                    )),
-                    CheckResult::Fail => Some((
-                        quick_junit::NonSuccessKind::Failure,
-                        "test failure with exit code",
-                    )),
-                    CheckResult::FailLeak => Some((
-                        quick_junit::NonSuccessKind::Failure,
-                        "test failure with exit code",
-                    )),
-                    CheckResult::Abort => {
-                        Some((quick_junit::NonSuccessKind::Failure, "test abort"))
-                    }
-                };
+                // Expected formats from nextest-runner/src/reporter/aggregator/junit.rs:
+                // - Fail (exit code, no leak): "test failure with exit code 101"
+                // - Fail (exit code, leaked):  "test failure with exit code 101, and leaked handles"
+                // - Abort (no leak):           "test abort"
+                // - Abort (leaked):            "test abort with leaked handles"
+                // - LeakFail:                  "test exited with code 0, but leaked handles so was marked failed"
+                // - Timeout:                   "test timeout" or "benchmark timeout"
+                let expected_junit: Option<(quick_junit::NonSuccessKind, &str)> =
+                    match expected_outcome.result {
+                        CheckResult::Pass | CheckResult::Leak => None,
+                        CheckResult::LeakFail => Some((
+                            quick_junit::NonSuccessKind::Error,
+                            "test exited with code 0, but leaked handles so was marked failed",
+                        )),
+                        CheckResult::Fail => {
+                            Some((quick_junit::NonSuccessKind::Failure, JUNIT_FAIL))
+                        }
+                        CheckResult::FailLeak => {
+                            Some((quick_junit::NonSuccessKind::Failure, JUNIT_FAIL_LEAK))
+                        }
+                        CheckResult::Abort => {
+                            Some((quick_junit::NonSuccessKind::Failure, JUNIT_ABORT))
+                        }
+                        CheckResult::Timeout => {
+                            let timeout_kind = if properties.contains(RunProperties::BENCHMARKS) {
+                                "benchmark timeout"
+                            } else {
+                                "test timeout"
+                            };
+                            Some((quick_junit::NonSuccessKind::Failure, timeout_kind))
+                        }
+                    };
 
                 match (expected_junit, &actual.non_success) {
                     (None, None) => {
                         // Both expected and actual were successful.
                     }
-                    (
-                        Some((expected_kind, expected_type_pattern)),
-                        Some((actual_kind, actual_type)),
-                    ) => {
+                    (Some((expected_kind, expected_type)), Some((actual_kind, actual_type))) => {
                         if expected_kind != *actual_kind {
                             panic!(
                                 "{}: expected JUnit kind {:?} but got {:?} (properties: {})\n\
@@ -793,28 +973,17 @@ fn verify_expected_in_junit(
                             );
                         }
 
-                        // For Fail/FailLeak, we check that the type string
-                        // contains the pattern (since we don't know the exact
-                        // exit code). For others, we check an exact match.
-                        let type_matches = if matches!(
-                            expected_outcome.result,
-                            CheckResult::Fail | CheckResult::FailLeak
-                        ) {
-                            actual_type.contains(expected_type_pattern)
-                                && (expected_outcome.result == CheckResult::FailLeak)
-                                    == actual_type.contains("leaked handles")
-                        } else {
-                            actual_type == expected_type_pattern
-                        };
+                        // Check if the actual type matches the expected string.
+                        let type_matches = expected_type == actual_type;
 
                         if !type_matches {
                             panic!(
-                                "{}: expected JUnit type containing {:?} but got {:?} \
+                                "{}: expected JUnit type {:?} but got {:?} \
                                  (properties: {})\n\
                                  JUnit path: {}\n\
                                  Expected result: {:?}",
                                 expected_outcome.id.full_name(),
-                                expected_type_pattern,
+                                expected_type,
                                 actual_type,
                                 debug_run_properties(properties),
                                 junit_path,
@@ -884,7 +1053,7 @@ fn verify_junit_in_expected(
     actual: &ActualJunitResults,
     expected: &ExpectedTestResults,
     junit_path: &Utf8Path,
-    properties: u64,
+    properties: RunProperties,
 ) {
     for outcome in &actual.tests {
         if !expected.should_run.contains_key(&outcome.id) {
