@@ -23,12 +23,14 @@
 //! `NEXTEST_BIN_EXE_cargo_nextest_dup`.
 
 use camino::{Utf8Path, Utf8PathBuf};
-use fixture_data::{models::RunProperty, nextest_tests::EXPECTED_TEST_SUITES};
+use fixture_data::{models::RunProperties, nextest_tests::EXPECTED_TEST_SUITES};
 use integration_tests::{
     env::set_env_vars,
     nextest_cli::{CargoNextestCli, CargoNextestOutput},
 };
-use nextest_metadata::{BuildPlatform, NextestExitCode, TestListSummary};
+use nextest_metadata::{
+    BuildPlatform, FilterMatch, MismatchReason, NextestExitCode, TestCaseName, TestListSummary,
+};
 use std::{borrow::Cow, fs::File, io::Write};
 use target_spec::{Platform, summaries::TargetFeaturesSummary};
 
@@ -145,6 +147,37 @@ fn test_list_full() {
         .output();
 
     check_list_full_output(&output.stdout, None);
+
+    // Test oneline format: output should be one test per line with format "binary_id test_name".
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "oneline",
+        ])
+        .output();
+
+    check_list_oneline_output(&output.stdout_as_str());
+
+    // Test auto format: when stdout is not a TTY, auto should produce oneline format.
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "auto",
+        ])
+        .output();
+
+    // Auto format should produce oneline when stdout is not a TTY.
+    check_list_oneline_output(&output.stdout_as_str());
 }
 
 #[test]
@@ -199,6 +232,23 @@ fn test_list_binaries_only() {
         .output();
 
     insta::assert_snapshot!(output.stderr_as_str());
+
+    // Test oneline format for binaries-only: output should be one binary per line.
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "oneline",
+            "--list-type",
+            "binaries-only",
+        ])
+        .output();
+
+    check_list_oneline_binaries_output(&output.stdout_as_str());
 }
 
 #[test]
@@ -447,7 +497,11 @@ fn test_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output_with_junit(&output.stderr, &p.junit_path("default"), 0);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperties::empty(),
+    );
 
     // --exact with nothing else should be the same as above.
     let output = CargoNextestCli::for_test()
@@ -468,7 +522,11 @@ fn test_run() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output_with_junit(&output.stderr, &p.junit_path("default"), 0);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperties::empty(),
+    );
 
     // Check the output with --skip.
     let output = CargoNextestCli::for_test()
@@ -492,7 +550,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithSkipCdylibFilter as u64,
+        RunProperties::WITH_SKIP_CDYLIB_FILTER,
     );
 
     // Equivalent filterset to the above.
@@ -516,7 +574,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithSkipCdylibFilter as u64,
+        RunProperties::WITH_SKIP_CDYLIB_FILTER,
     );
 
     // Check the output with --exact.
@@ -537,7 +595,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithMultiplyTwoExactFilter as u64,
+        RunProperties::WITH_MULTIPLY_TWO_EXACT_FILTER,
     );
 
     // Equivalent filterset to the above.
@@ -556,7 +614,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithMultiplyTwoExactFilter as u64,
+        RunProperties::WITH_MULTIPLY_TWO_EXACT_FILTER,
     );
 
     // Check the output with --exact and --skip.
@@ -581,7 +639,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
+        RunProperties::WITH_SKIP_CDYLIB_FILTER | RunProperties::WITH_MULTIPLY_TWO_EXACT_FILTER,
     );
 
     // Equivalent filterset to the above.
@@ -601,7 +659,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
+        RunProperties::WITH_SKIP_CDYLIB_FILTER | RunProperties::WITH_MULTIPLY_TWO_EXACT_FILTER,
     );
 
     // Another equivalent.
@@ -624,7 +682,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
+        RunProperties::WITH_SKIP_CDYLIB_FILTER | RunProperties::WITH_MULTIPLY_TWO_EXACT_FILTER,
     );
 
     // Yet another equivalent.
@@ -648,7 +706,7 @@ fn test_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("default"),
-        RunProperty::WithSkipCdylibFilter as u64 | RunProperty::WithMultiplyTwoExactFilter as u64,
+        RunProperties::WITH_SKIP_CDYLIB_FILTER | RunProperties::WITH_MULTIPLY_TWO_EXACT_FILTER,
     );
 }
 
@@ -675,7 +733,137 @@ fn test_run_after_build() {
         Some(NextestExitCode::TEST_RUN_FAILED),
         "correct exit code for command\n{output}"
     );
-    check_run_output_with_junit(&output.stderr, &p.junit_path("default"), 0);
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperties::empty(),
+    );
+}
+
+/// Test that per-benchmark override for bench.slow-timeout is respected.
+///
+/// The profile has a 30-year bench.slow-timeout at the profile level, but an
+/// override that sets a 1s timeout for `bench_slow_timeout`. The benchmark
+/// sleeps for 4 seconds, so it should time out.
+#[test]
+fn test_bench_override_slow_timeout() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "bench",
+            "--profile",
+            "with-bench-override",
+            // Set the dev profile here to avoid a rebuild.
+            "--cargo-profile",
+            "dev",
+            "--run-ignored",
+            "only",
+            "-E",
+            "test(=bench_slow_timeout)",
+        ])
+        .unchecked(true)
+        .output();
+
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "correct exit code for command (benchmark should time out)\n{output}",
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-bench-override"),
+        RunProperties::BENCH_OVERRIDE_TIMEOUT
+            | RunProperties::BENCHMARKS
+            | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Test that bench.slow-timeout causes benchmarks to time out.
+#[test]
+fn test_bench_termination() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "bench",
+            "--profile",
+            "with-bench-termination",
+            // Use test profile to avoid rebuilding: `cargo nextest bench`
+            // defaults to the `bench` cargo profile, but the fixture is already
+            // built with `test`.
+            "--cargo-profile",
+            "test",
+            "--run-ignored",
+            "only",
+            "-E",
+            "test(=bench_slow_timeout)",
+        ])
+        .unchecked(true)
+        .output();
+
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "correct exit code for command (benchmark should time out)\n{output}",
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-bench-termination"),
+        RunProperties::BENCH_TERMINATION
+            | RunProperties::BENCHMARKS
+            | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Test that benchmarks ignore regular slow-timeout and only use bench.slow-timeout.
+///
+/// The benchmark sleeps for 2 seconds. The profile has slow-timeout = 1s but no
+/// bench.slow-timeout (defaults to 30 years). If benchmarks incorrectly used
+/// slow-timeout, this would fail. But it passes because benchmarks use
+/// bench.slow-timeout.
+#[test]
+fn test_bench_ignores_test_slow_timeout() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "bench",
+            "--profile",
+            "with-test-termination-only",
+            // Use test profile to avoid rebuilding: `cargo nextest bench`
+            // defaults to the `bench` cargo profile, but the fixture is already
+            // built with `test`.
+            "--cargo-profile",
+            "test",
+            "--run-ignored",
+            "only",
+            "-E",
+            "test(=bench_slow_timeout)",
+        ])
+        .output();
+
+    // The benchmark should pass (not time out), even though slow-timeout is 1s,
+    // because benchmarks only use bench.slow-timeout which defaults to 30
+    // years.
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::OK),
+        "correct exit code for command (benchmark should succeed)\n{output}",
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-test-termination-only"),
+        RunProperties::BENCH_IGNORES_TEST_TIMEOUT
+            | RunProperties::BENCHMARKS
+            | RunProperties::SKIP_SUMMARY_CHECK,
+    );
 }
 
 #[test]
@@ -737,7 +925,7 @@ fn test_relocated_run() {
     check_run_output_with_junit(
         &output.stderr,
         &p2.junit_path("default"),
-        RunProperty::Relocated as u64,
+        RunProperties::RELOCATED,
     );
 }
 
@@ -887,7 +1075,7 @@ fn test_archive_with_build_filter() {
         }
         run_archive_with_args(
             &archive_file,
-            RunProperty::Relocated as u64,
+            RunProperties::RELOCATED,
             NextestExitCode::TEST_RUN_FAILED,
         );
     });
@@ -912,7 +1100,7 @@ fn test_archive_with_build_filter() {
         }
         run_archive_with_args(
             &archive_file,
-            RunProperty::SkipSummaryCheck as u64 | RunProperty::ExpectNoBinaries as u64,
+            RunProperties::SKIP_SUMMARY_CHECK | RunProperties::EXPECT_NO_BINARIES,
             NextestExitCode::NO_TESTS_RUN,
         );
     });
@@ -937,7 +1125,7 @@ fn test_archive_with_build_filter() {
         );
         run_archive_with_args(
             &archive_file,
-            RunProperty::CdyLibExamplePackageFilter as u64 | RunProperty::SkipSummaryCheck as u64,
+            RunProperties::CDYLIB_EXAMPLE_PACKAGE_FILTER | RunProperties::SKIP_SUMMARY_CHECK,
             NextestExitCode::OK,
         );
     });
@@ -1101,14 +1289,14 @@ fn create_archive_with_args(
 fn run_archive(archive_file: &Utf8Path) -> (TempProject, Utf8PathBuf) {
     run_archive_with_args(
         archive_file,
-        RunProperty::Relocated as u64,
+        RunProperties::RELOCATED,
         NextestExitCode::TEST_RUN_FAILED,
     )
 }
 
 fn run_archive_with_args(
     archive_file: &Utf8Path,
-    run_property: u64,
+    run_property: RunProperties,
     expected_exit_code: i32,
 ) -> (TempProject, Utf8PathBuf) {
     let p2 = TempProject::new().unwrap();
@@ -1138,6 +1326,34 @@ fn run_archive_with_args(
     (p2, extract_to.join("target"))
 }
 
+#[test]
+fn test_bench() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "bench",
+            // Set the dev profile here to avoid a rebuild.
+            "--cargo-profile",
+            "dev",
+            "--no-capture",
+        ])
+        .unchecked(true)
+        .output();
+    assert_eq!(
+        output.exit_status.code(),
+        Some(0),
+        "correct exit code for command\n{output}",
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperties::BENCHMARKS,
+    );
+}
+
 #[test]
 fn test_show_config_test_groups() {
     set_env_vars();
@@ -1436,7 +1652,7 @@ fn test_run_with_default_filter() {
     check_run_output_with_junit(
         &output.stderr,
         &p.junit_path("with-default-filter"),
-        RunProperty::WithDefaultFilter as u64,
+        RunProperties::WITH_DEFAULT_FILTER,
     );
 }
 
@@ -1769,3 +1985,515 @@ fn test_rustc_version_verbose_errors() {
         );
     }
 }
+
+/// Test that filterset expressions combined with string filters work correctly.
+#[test]
+fn test_filterset_with_string_filters() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    // Expression: test(test_multiply_two) | test(=tests::call_dylib_add_two)
+    // String filters: call_dylib_add_two, test_flaky_mod_4
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "json",
+            "-E",
+            "test(test_multiply_two) | test(=tests::call_dylib_add_two)",
+            "--",
+            "call_dylib_add_two",
+            "test_flaky_mod_4",
+        ])
+        .output();
+
+    let summary: TestListSummary =
+        serde_json::from_slice(&output.stdout).expect("failed to parse test list JSON");
+
+    // Verify specific test cases:
+    for (binary_id, suite) in &summary.rust_suites {
+        for (test_name, test_info) in &suite.test_cases {
+            let full_name = format!("{} {}", binary_id, test_name);
+
+            if *test_name == TestCaseName::new("tests::call_dylib_add_two") {
+                // Matches both expression and string filter.
+                assert!(
+                    test_info.filter_match.is_match(),
+                    "{full_name}: expected match, got {:?}",
+                    test_info.filter_match
+                );
+            } else if test_name.contains("test_multiply_two") {
+                // Matches expression but not string filter.
+                assert_eq!(
+                    test_info.filter_match,
+                    FilterMatch::Mismatch {
+                        reason: MismatchReason::String
+                    },
+                    "{full_name}: expected String mismatch"
+                );
+            } else if test_name.contains("test_flaky_mod_4") {
+                // Matches string filter but not expression.
+                assert_eq!(
+                    test_info.filter_match,
+                    FilterMatch::Mismatch {
+                        reason: MismatchReason::Expression
+                    },
+                    "{full_name}: expected Expression mismatch"
+                );
+            } else {
+                // Should not match.
+                assert!(
+                    !test_info.filter_match.is_match(),
+                    "{full_name}: expected no match, got {:?}",
+                    test_info.filter_match
+                );
+            }
+        }
+    }
+}
+
+/// Test that filterset expressions without string filters work correctly.
+#[test]
+fn test_filterset_without_string_filters() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    // Expression only, no string filters.
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "json",
+            "-E",
+            "test(test_multiply_two) | test(=tests::call_dylib_add_two)",
+        ])
+        .output();
+
+    let summary: TestListSummary =
+        serde_json::from_slice(&output.stdout).expect("failed to parse test list JSON");
+
+    for (binary_id, suite) in &summary.rust_suites {
+        for (test_name, test_info) in &suite.test_cases {
+            let full_name = format!("{} {}", binary_id, test_name);
+
+            if test_name.contains("test_multiply_two")
+                || *test_name == TestCaseName::new("tests::call_dylib_add_two")
+            {
+                assert!(
+                    test_info.filter_match.is_match(),
+                    "{full_name}: expected match, got {:?}",
+                    test_info.filter_match
+                );
+            } else {
+                assert!(
+                    !test_info.filter_match.is_match(),
+                    "{full_name}: expected no match, got {:?}",
+                    test_info.filter_match
+                );
+            }
+        }
+    }
+}
+
+/// Test that string filters without filterset expressions work correctly.
+#[test]
+fn test_string_filters_without_filterset() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    // String filters only, no expression.
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "json",
+            "--",
+            "test_multiply_two",
+            "tests::call_dylib_add_two",
+        ])
+        .output();
+
+    let summary: TestListSummary =
+        serde_json::from_slice(&output.stdout).expect("failed to parse test list JSON");
+
+    for (binary_id, suite) in &summary.rust_suites {
+        for (test_name, test_info) in &suite.test_cases {
+            let full_name = format!("{} {}", binary_id, test_name);
+
+            if test_name.contains("test_multiply_two")
+                || test_name.contains("tests::call_dylib_add_two")
+            {
+                assert!(
+                    test_info.filter_match.is_match(),
+                    "{full_name}: expected match, got {:?}",
+                    test_info.filter_match
+                );
+            } else {
+                assert!(
+                    !test_info.filter_match.is_match(),
+                    "{full_name}: expected no match, got {:?}",
+                    test_info.filter_match
+                );
+            }
+        }
+    }
+}
+
+/// Test that `--run-ignored only` runs only ignored tests.
+#[test]
+fn test_run_ignored() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--workspace",
+            "--all-targets",
+            "--run-ignored",
+            "only",
+            "-E",
+            // Filter out slow timeout tests to avoid long test times.
+            "not test(slow_timeout)",
+        ])
+        .unchecked(true)
+        .output();
+
+    // The run should fail because some ignored tests fail (e.g., test_ignored_fail).
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "correct exit code for command\n{output}"
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperties::RUN_IGNORED_ONLY,
+    );
+}
+
+/// Test that timed out tests with `on-timeout=pass` are not retried.
+///
+/// The profile `with-timeout-retries-success` has:
+/// - `retries = 2`
+/// - `slow-timeout = { period = "500ms", terminate-after = 2, on-timeout = "pass" }`
+///
+/// Tests that time out with `on-timeout = "pass"` should be marked as passed.
+#[test]
+fn test_timeout_with_retries() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--profile",
+            "with-timeout-retries-success",
+            "--run-ignored",
+            "only",
+            "-E",
+            // Only run the slow timeout tests (not the flaky one).
+            "test(/^test_slow_timeout/)",
+        ])
+        .output();
+
+    // Should succeed because on-timeout=pass marks timed out tests as passed.
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::OK),
+        "timed out tests with on-timeout=pass should succeed\n{output}"
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-timeout-retries-success"),
+        RunProperties::TIMEOUT_RETRIES_PASS | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Test that a flaky test that times out on the 3rd attempt is handled correctly.
+///
+/// The test `test_flaky_slow_timeout_mod_3`:
+/// - Fails on attempts 1 and 2 (attempt % 3 != 0)
+/// - Times out on attempt 3 (attempt % 3 == 0, so it sleeps until timeout)
+///
+/// With the `with-timeout-retries-success` profile (retries=2, on-timeout=pass),
+/// the test should be marked as flaky (failed twice, then passed via timeout).
+#[test]
+fn test_timeout_with_flaky() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--profile",
+            "with-timeout-retries-success",
+            "--run-ignored",
+            "only",
+            "-E",
+            "test(test_flaky_slow_timeout_mod_3)",
+        ])
+        .output();
+
+    // Should succeed because the test eventually passes (via timeout with on-timeout=pass).
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::OK),
+        "flaky test that eventually times out with on-timeout=pass should succeed\n{output}"
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-timeout-retries-success"),
+        RunProperties::TIMEOUT_RETRIES_FLAKY | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Test that retries work correctly with the with-retries profile.
+///
+/// The `with-retries` profile has:
+/// - `retries = 2` (default)
+/// - Override for test_flaky_mod_6: `retries = 5`
+/// - Override for test_flaky_mod_4: `retries = 4`
+///
+/// Flaky tests should eventually pass after the configured number of retries.
+#[test]
+fn test_retries() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--workspace",
+            "--all-targets",
+            "--profile",
+            "with-retries",
+        ])
+        .unchecked(true)
+        .output();
+
+    // Should fail because some tests fail even after retries.
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "test run should fail due to failing tests\n{output}"
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-retries"),
+        RunProperties::WITH_RETRIES | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Test that tests time out correctly with the with-termination profile.
+///
+/// The `with-termination` profile has slow-timeout configured such that tests
+/// time out after 2 seconds. The test_slow_timeout* tests sleep for longer than
+/// this and should all be terminated.
+#[test]
+fn test_termination() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--profile",
+            "with-termination",
+            "--run-ignored",
+            "only",
+            "-E",
+            "test(/^test_slow_timeout/)",
+        ])
+        .unchecked(true)
+        .output();
+
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "tests should time out and fail\n{output}"
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-termination"),
+        RunProperties::WITH_TERMINATION | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Test that on-timeout = "pass" works correctly with the with-timeout-success profile.
+///
+/// The `with-timeout-success` profile has an override for test_slow_timeout that
+/// sets on-timeout = "pass", so it should pass when it times out. The other
+/// test_slow_timeout* tests should fail with timeout.
+#[test]
+fn test_override_timeout_result() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--profile",
+            "with-timeout-success",
+            "--run-ignored",
+            "only",
+            "-E",
+            "test(/^test_slow_timeout/)",
+        ])
+        .unchecked(true)
+        .output();
+
+    // Should fail because test_slow_timeout_2 and test_slow_timeout_subprocess
+    // time out without on-timeout = "pass".
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "some tests should time out and fail\n{output}"
+    );
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("with-timeout-success"),
+        RunProperties::WITH_TIMEOUT_SUCCESS | RunProperties::SKIP_SUMMARY_CHECK,
+    );
+}
+
+/// Returns the CARGO_TARGET_<triple>_RUNNER env var name for the current platform.
+fn current_runner_env_var() -> String {
+    let platform = Platform::build_target().expect("current platform is known to target-spec");
+    let triple = platform.triple_str().to_uppercase().replace('-', "_");
+    format!("CARGO_TARGET_{triple}_RUNNER")
+}
+
+/// Test that listing works correctly with a target runner set.
+#[test]
+fn test_listing_with_target_runner() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    // Get the passthrough binary path.
+    let passthrough = std::env::var("NEXTEST_BIN_EXE_passthrough")
+        .expect("passthrough binary should be available");
+
+    // First, list without target runner to get baseline counts.
+    let baseline_output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "json",
+        ])
+        .output();
+
+    let baseline: TestListSummary =
+        serde_json::from_slice(&baseline_output.stdout).expect("parse baseline JSON");
+
+    // Now list with target runner set.
+    let runner_env = current_runner_env_var();
+    let runner_value = format!("{passthrough} --ensure-this-arg-is-sent");
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "list",
+            "--workspace",
+            "--all-targets",
+            "--message-format",
+            "json",
+        ])
+        .env(&runner_env, &runner_value)
+        .output();
+
+    let with_runner: TestListSummary =
+        serde_json::from_slice(&output.stdout).expect("parse with-runner JSON");
+
+    // Verify the counts are the same.
+    assert_eq!(
+        baseline.rust_suites.len(),
+        with_runner.rust_suites.len(),
+        "binary counts should match"
+    );
+
+    let baseline_test_count: usize = baseline
+        .rust_suites
+        .values()
+        .map(|s| s.test_cases.len())
+        .sum();
+    let with_runner_test_count: usize = with_runner
+        .rust_suites
+        .values()
+        .map(|s| s.test_cases.len())
+        .sum();
+    assert_eq!(
+        baseline_test_count, with_runner_test_count,
+        "test counts should match"
+    );
+}
+
+/// Test that running tests works correctly with a target runner set.
+#[test]
+fn test_run_with_target_runner() {
+    set_env_vars();
+    let p = TempProject::new().unwrap();
+
+    // Get the passthrough binary path.
+    let passthrough = std::env::var("NEXTEST_BIN_EXE_passthrough")
+        .expect("passthrough binary should be available");
+
+    let runner_env = current_runner_env_var();
+    let runner_value = format!("{passthrough} --ensure-this-arg-is-sent");
+
+    let output = CargoNextestCli::for_test()
+        .args([
+            "--manifest-path",
+            p.manifest_path().as_str(),
+            "run",
+            "--workspace",
+            "--all-targets",
+        ])
+        .env(&runner_env, &runner_value)
+        .unchecked(true)
+        .output();
+
+    // Should fail because some tests fail (same as without target runner).
+    assert_eq!(
+        output.exit_status.code(),
+        Some(NextestExitCode::TEST_RUN_FAILED),
+        "correct exit code for command\n{output}"
+    );
+
+    // On Unix, segfaults aren't passed through correctly by the passthrough runner
+    // (they show as regular failures), so we use a special property flag.
+    check_run_output_with_junit(
+        &output.stderr,
+        &p.junit_path("default"),
+        RunProperties::WITH_TARGET_RUNNER,
+    );
+}
