@@ -1,15 +1,16 @@
 // Copyright (c) The nextest Contributors
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use super::{NextestVersionDeserialize, ToolConfigFile};
+use super::{NextestVersionDeserialize, ToolConfigFile, ToolName};
 use crate::{
     config::{
         core::ConfigExperimental,
         elements::{
-            ArchiveConfig, CustomTestGroup, DefaultJunitImpl, GlobalTimeout, JunitConfig,
-            JunitImpl, LeakTimeout, MaxFail, RetryPolicy, SlowTimeout, TestGroup, TestGroupConfig,
-            TestThreads, ThreadsRequired, deserialize_fail_fast, deserialize_leak_timeout,
-            deserialize_retry_policy, deserialize_slow_timeout,
+            ArchiveConfig, BenchConfig, CustomTestGroup, DefaultBenchConfig, DefaultJunitImpl,
+            GlobalTimeout, Inherits, JunitConfig, JunitImpl, JunitSettings, LeakTimeout, MaxFail,
+            RetryPolicy, SlowTimeout, TestGroup, TestGroupConfig, TestThreads, ThreadsRequired,
+            deserialize_fail_fast, deserialize_leak_timeout, deserialize_retry_policy,
+            deserialize_slow_timeout,
         },
         overrides::{
             CompiledByProfile, CompiledData, CompiledDefaultFilter, DeserializedOverride,
@@ -21,14 +22,16 @@ use crate::{
         },
     },
     errors::{
-        ConfigParseError, ConfigParseErrorKind, ProfileListScriptUsesRunFiltersError,
-        ProfileNotFound, ProfileScriptErrors, ProfileUnknownScriptError,
-        ProfileWrongConfigScriptTypeError, UnknownTestGroupError, provided_by_tool,
+        ConfigParseError, ConfigParseErrorKind, InheritsError,
+        ProfileListScriptUsesRunFiltersError, ProfileNotFound, ProfileScriptErrors,
+        ProfileUnknownScriptError, ProfileWrongConfigScriptTypeError, UnknownTestGroupError,
+        provided_by_tool,
     },
     helpers::plural,
     list::TestList,
     platform::BuildPlatforms,
     reporter::{FinalStatusLevel, StatusLevel, TestOutputDisplay},
+    run_mode::NextestRunMode,
 };
 use camino::{Utf8Path, Utf8PathBuf};
 use config::{
@@ -37,6 +40,7 @@ use config::{
 use iddqd::IdOrdMap;
 use indexmap::IndexMap;
 use nextest_filtering::{BinaryQuery, EvalContext, Filterset, ParseContext, TestQuery};
+use petgraph::{Directed, Graph, algo::scc::kosaraju_scc, graph::NodeIndex};
 use serde::Deserialize;
 use std::{
     collections::{BTreeMap, BTreeSet, HashMap, hash_map},
@@ -54,7 +58,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         unknown: &BTreeSet<String>,
     );
 
@@ -63,7 +67,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profiles: &[&str],
     );
 
@@ -72,7 +76,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
     );
 
     /// Handle warning about empty script sections with neither setup nor
@@ -81,7 +85,7 @@ pub trait ConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profile_name: &str,
         empty_count: usize,
     );
@@ -95,7 +99,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         unknown: &BTreeSet<String>,
     ) {
         let mut unknown_str = String::new();
@@ -124,7 +128,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profiles: &[&str],
     ) {
         warn!(
@@ -144,7 +148,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
     ) {
         warn!(
             "in config file {}{}, [script.*] is deprecated and will be removed in a \
@@ -160,7 +164,7 @@ impl ConfigWarnings for DefaultConfigWarnings {
         &mut self,
         config_file: &Utf8Path,
         workspace_root: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         profile_name: &str,
         empty_count: usize,
     ) {
@@ -371,6 +375,9 @@ impl NextestConfig {
 
         let mut known_groups = BTreeSet::new();
         let mut known_scripts = IdOrdMap::new();
+        // Track known profiles for inheritance validation. Profiles can only inherit
+        // from profiles defined in the same file or in previously loaded (lower priority) files.
+        let mut known_profiles = BTreeSet::new();
 
         // Next, merge in tool configs.
         for ToolConfigFile { config_file, tool } in tool_config_files_rev {
@@ -386,6 +393,7 @@ impl NextestConfig {
                 warnings,
                 &mut known_groups,
                 &mut known_scripts,
+                &mut known_profiles,
             )?;
 
             // This is the final, composite builder used at the end.
@@ -413,6 +421,7 @@ impl NextestConfig {
             warnings,
             &mut known_groups,
             &mut known_scripts,
+            &mut known_profiles,
         )?;
 
         composite_builder = composite_builder.add_source(source);
@@ -420,7 +429,7 @@ impl NextestConfig {
         // The unknown set is ignored here because any values in it have already been reported in
         // deserialize_individual_config.
         let (config, _unknown) = Self::build_and_deserialize_config(&composite_builder)
-            .map_err(|kind| ConfigParseError::new(config_file, None, kind))?;
+            .map_err(|kind| ConfigParseError::new(&config_file, None, kind))?;
 
         // Reverse all the compiled data at the end.
         compiled.default.reverse();
@@ -436,13 +445,14 @@ impl NextestConfig {
         pcx: &ParseContext<'_>,
         workspace_root: &Utf8Path,
         config_file: &Utf8Path,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         source: File<FileSourceFile, FileFormat>,
         compiled_out: &mut CompiledByProfile,
         experimental: &BTreeSet<ConfigExperimental>,
         warnings: &mut impl ConfigWarnings,
         known_groups: &mut BTreeSet<CustomTestGroup>,
         known_scripts: &mut IdOrdMap<ScriptInfo>,
+        known_profiles: &mut BTreeSet<String>,
     ) -> Result<(), ConfigParseError> {
         // Try building default builder + this file to get good error attribution and handle
         // overrides additively.
@@ -463,7 +473,7 @@ impl NextestConfig {
                     group
                         .as_identifier()
                         .tool_components()
-                        .is_some_and(|(tool_name, _)| tool_name == tool)
+                        .is_some_and(|(tool_name, _)| tool_name == tool.as_str())
                 } else {
                     // If a tool is not specified, it must *not* be a tool identifier.
                     !group.as_identifier().is_tool_identifier()
@@ -538,7 +548,7 @@ impl NextestConfig {
                     script
                         .as_identifier()
                         .tool_components()
-                        .is_some_and(|(tool_name, _)| tool_name == tool)
+                        .is_some_and(|(tool_name, _)| tool_name == tool.as_str())
                 } else {
                     // If a tool is not specified, it must *not* be a tool identifier.
                     !script.as_identifier().is_tool_identifier()
@@ -575,6 +585,20 @@ impl NextestConfig {
             );
         }
 
+        // Check that the profiles correctly use the inherits setting.
+        // Profiles can only inherit from profiles in the same file or in previously
+        // loaded (lower priority) files.
+        this_config
+            .sanitize_profile_inherits(known_profiles)
+            .map_err(|kind| ConfigParseError::new(config_file, tool, kind))?;
+
+        // Add this file's profiles to known_profiles for subsequent files.
+        known_profiles.extend(
+            this_config
+                .other_profiles()
+                .map(|(name, _)| name.to_owned()),
+        );
+
         // Compile the overrides for this file.
         let this_compiled = CompiledByProfile::new(pcx, &this_config)
             .map_err(|kind| ConfigParseError::new(config_file, tool, kind))?;
@@ -794,6 +818,9 @@ impl NextestConfig {
     fn make_profile(&self, name: &str) -> Result<EarlyProfile<'_>, ProfileNotFound> {
         let custom_profile = self.inner.get_profile(name)?;
 
+        // Resolve the inherited profile into a profile chain
+        let inheritance_chain = self.inner.resolve_inheritance_chain(name)?;
+
         // The profile was found: construct it.
         let mut store_dir = self.workspace_root.join(&self.inner.store.dir);
         store_dir.push(name);
@@ -809,6 +836,7 @@ impl NextestConfig {
             store_dir,
             default_profile: &self.inner.default_profile,
             custom_profile,
+            inheritance_chain,
             test_groups: &self.inner.test_groups,
             scripts: &self.inner.scripts,
             compiled_data,
@@ -874,6 +902,7 @@ pub struct EarlyProfile<'cfg> {
     store_dir: Utf8PathBuf,
     default_profile: &'cfg DefaultProfileImpl,
     custom_profile: Option<&'cfg CustomProfileImpl>,
+    inheritance_chain: Vec<&'cfg CustomProfileImpl>,
     test_groups: &'cfg BTreeMap<CustomTestGroup, TestGroupConfig>,
     // This is ordered because the scripts are used in the order they're defined.
     scripts: &'cfg ScriptConfig,
@@ -924,6 +953,7 @@ impl<'cfg> EarlyProfile<'cfg> {
             store_dir: self.store_dir,
             default_profile: self.default_profile,
             custom_profile: self.custom_profile,
+            inheritance_chain: self.inheritance_chain,
             scripts: self.scripts,
             test_groups: self.test_groups,
             compiled_data,
@@ -941,6 +971,7 @@ pub struct EvaluatableProfile<'cfg> {
     store_dir: Utf8PathBuf,
     default_profile: &'cfg DefaultProfileImpl,
     custom_profile: Option<&'cfg CustomProfileImpl>,
+    inheritance_chain: Vec<&'cfg CustomProfileImpl>,
     test_groups: &'cfg BTreeMap<CustomTestGroup, TestGroupConfig>,
     // This is ordered because the scripts are used in the order they're defined.
     scripts: &'cfg ScriptConfig,
@@ -951,6 +982,65 @@ pub struct EvaluatableProfile<'cfg> {
     resolved_default_filter: CompiledDefaultFilter,
 }
 
+/// These macros return a specific config field from an EvaluatableProfile,
+/// checking in order: custom profile, inheritance chain, then default profile.
+macro_rules! profile_field {
+    ($eval_prof:ident.$field:ident) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$field)
+            .unwrap_or($eval_prof.default_profile.$field)
+    };
+    ($eval_prof:ident.$nested:ident.$field:ident) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$nested.$field)
+            .unwrap_or($eval_prof.default_profile.$nested.$field)
+    };
+    // Variant for method calls with arguments.
+    ($eval_prof:ident.$method:ident($($arg:expr),*)) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$method($($arg),*))
+            .unwrap_or_else(|| $eval_prof.default_profile.$method($($arg),*))
+    };
+}
+macro_rules! profile_field_from_ref {
+    ($eval_prof:ident.$field:ident.$ref_func:ident()) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$field.$ref_func())
+            .unwrap_or(&$eval_prof.default_profile.$field)
+    };
+    ($eval_prof:ident.$nested:ident.$field:ident.$ref_func:ident()) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$nested.$field.$ref_func())
+            .unwrap_or(&$eval_prof.default_profile.$nested.$field)
+    };
+}
+// Variant for fields where both custom and default are Option.
+macro_rules! profile_field_optional {
+    ($eval_prof:ident.$nested:ident.$field:ident.$ref_func:ident()) => {
+        $eval_prof
+            .custom_profile
+            .iter()
+            .chain($eval_prof.inheritance_chain.iter())
+            .find_map(|p| p.$nested.$field.$ref_func())
+            .or($eval_prof.default_profile.$nested.$field.$ref_func())
+    };
+}
+
 impl<'cfg> EvaluatableProfile<'cfg> {
     /// Returns the name of the profile.
     pub fn name(&self) -> &str {
@@ -986,94 +1076,68 @@ impl<'cfg> EvaluatableProfile<'cfg> {
 
     /// Returns the retry count for this profile.
     pub fn retries(&self) -> RetryPolicy {
-        self.custom_profile
-            .and_then(|profile| profile.retries)
-            .unwrap_or(self.default_profile.retries)
+        profile_field!(self.retries)
     }
 
     /// Returns the number of threads to run against for this profile.
     pub fn test_threads(&self) -> TestThreads {
-        self.custom_profile
-            .and_then(|profile| profile.test_threads)
-            .unwrap_or(self.default_profile.test_threads)
+        profile_field!(self.test_threads)
     }
 
     /// Returns the number of threads required for each test.
     pub fn threads_required(&self) -> ThreadsRequired {
-        self.custom_profile
-            .and_then(|profile| profile.threads_required)
-            .unwrap_or(self.default_profile.threads_required)
+        profile_field!(self.threads_required)
     }
 
     /// Returns extra arguments to be passed to the test binary at runtime.
     pub fn run_extra_args(&self) -> &'cfg [String] {
-        self.custom_profile
-            .and_then(|profile| profile.run_extra_args.as_deref())
-            .unwrap_or(&self.default_profile.run_extra_args)
+        profile_field_from_ref!(self.run_extra_args.as_deref())
     }
 
     /// Returns the time after which tests are treated as slow for this profile.
-    pub fn slow_timeout(&self) -> SlowTimeout {
-        self.custom_profile
-            .and_then(|profile| profile.slow_timeout)
-            .unwrap_or(self.default_profile.slow_timeout)
+    pub fn slow_timeout(&self, run_mode: NextestRunMode) -> SlowTimeout {
+        profile_field!(self.slow_timeout(run_mode))
     }
 
     /// Returns the time after which we should stop running tests.
-    pub fn global_timeout(&self) -> GlobalTimeout {
-        self.custom_profile
-            .and_then(|profile| profile.global_timeout)
-            .unwrap_or(self.default_profile.global_timeout)
+    pub fn global_timeout(&self, run_mode: NextestRunMode) -> GlobalTimeout {
+        profile_field!(self.global_timeout(run_mode))
     }
 
     /// Returns the time after which a child process that hasn't closed its handles is marked as
     /// leaky.
     pub fn leak_timeout(&self) -> LeakTimeout {
-        self.custom_profile
-            .and_then(|profile| profile.leak_timeout)
-            .unwrap_or(self.default_profile.leak_timeout)
+        profile_field!(self.leak_timeout)
     }
 
     /// Returns the test status level.
     pub fn status_level(&self) -> StatusLevel {
-        self.custom_profile
-            .and_then(|profile| profile.status_level)
-            .unwrap_or(self.default_profile.status_level)
+        profile_field!(self.status_level)
     }
 
     /// Returns the test status level at the end of the run.
     pub fn final_status_level(&self) -> FinalStatusLevel {
-        self.custom_profile
-            .and_then(|profile| profile.final_status_level)
-            .unwrap_or(self.default_profile.final_status_level)
+        profile_field!(self.final_status_level)
     }
 
     /// Returns the failure output config for this profile.
     pub fn failure_output(&self) -> TestOutputDisplay {
-        self.custom_profile
-            .and_then(|profile| profile.failure_output)
-            .unwrap_or(self.default_profile.failure_output)
+        profile_field!(self.failure_output)
     }
 
     /// Returns the failure output config for this profile.
     pub fn success_output(&self) -> TestOutputDisplay {
-        self.custom_profile
-            .and_then(|profile| profile.success_output)
-            .unwrap_or(self.default_profile.success_output)
+        profile_field!(self.success_output)
     }
 
     /// Returns the max-fail config for this profile.
     pub fn max_fail(&self) -> MaxFail {
-        self.custom_profile
-            .and_then(|profile| profile.max_fail)
-            .unwrap_or(self.default_profile.max_fail)
+        profile_field!(self.max_fail)
     }
 
     /// Returns the archive configuration for this profile.
     pub fn archive_config(&self) -> &'cfg ArchiveConfig {
-        self.custom_profile
-            .and_then(|profile| profile.archive.as_ref())
-            .unwrap_or(&self.default_profile.archive)
+        profile_field_from_ref!(self.archive.as_ref())
     }
 
     /// Returns the list of setup scripts.
@@ -1087,25 +1151,40 @@ impl<'cfg> EvaluatableProfile<'cfg> {
     }
 
     /// Returns settings for individual tests.
-    pub fn settings_for(&self, query: &TestQuery<'_>) -> TestSettings<'_> {
-        TestSettings::new(self, query)
+    pub fn settings_for(
+        &self,
+        run_mode: NextestRunMode,
+        query: &TestQuery<'_>,
+    ) -> TestSettings<'_> {
+        TestSettings::new(self, run_mode, query)
     }
 
     /// Returns override settings for individual tests, with sources attached.
     pub(crate) fn settings_with_source_for(
         &self,
+        run_mode: NextestRunMode,
         query: &TestQuery<'_>,
     ) -> TestSettings<'_, SettingSource<'_>> {
-        TestSettings::new(self, query)
+        TestSettings::new(self, run_mode, query)
     }
 
     /// Returns the JUnit configuration for this profile.
     pub fn junit(&self) -> Option<JunitConfig<'cfg>> {
-        JunitConfig::new(
-            self.store_dir(),
-            self.custom_profile.map(|p| &p.junit),
-            &self.default_profile.junit,
-        )
+        let settings = JunitSettings {
+            path: profile_field_optional!(self.junit.path.as_deref()),
+            report_name: profile_field_from_ref!(self.junit.report_name.as_deref()),
+            store_success_output: profile_field!(self.junit.store_success_output),
+            store_failure_output: profile_field!(self.junit.store_failure_output),
+        };
+        JunitConfig::new(self.store_dir(), settings)
+    }
+
+    /// Returns the profile that this profile inherits from.
+    pub fn inherits(&self) -> Option<&str> {
+        if let Some(custom_profile) = self.custom_profile {
+            return custom_profile.inherits();
+        }
+        None
     }
 
     #[cfg(test)]
@@ -1154,6 +1233,199 @@ impl NextestConfigImpl {
             .iter()
             .map(|(key, value)| (key.as_str(), value))
     }
+
+    /// Resolve a profile's inheritance chain (ancestors only, not including the
+    /// profile itself).
+    ///
+    /// Returns the chain ordered from immediate parent to furthest ancestor.
+    /// Cycles are assumed to have been checked by `sanitize_profile_inherits()`.
+    fn resolve_inheritance_chain(
+        &self,
+        profile_name: &str,
+    ) -> Result<Vec<&CustomProfileImpl>, ProfileNotFound> {
+        let mut chain = Vec::new();
+
+        // Start from the profile's parent, not the profile itself (the profile
+        // is already available via custom_profile).
+        let mut curr = self
+            .get_profile(profile_name)?
+            .and_then(|p| p.inherits.as_deref());
+
+        while let Some(name) = curr {
+            let profile = self.get_profile(name)?;
+            if let Some(profile) = profile {
+                chain.push(profile);
+                curr = profile.inherits.as_deref();
+            } else {
+                // Reached the default profile -- stop.
+                break;
+            }
+        }
+
+        Ok(chain)
+    }
+
+    /// Sanitize inherits settings on default and custom profiles.
+    ///
+    /// `known_profiles` contains profiles from previously loaded (lower priority) files.
+    /// A profile can inherit from profiles in the same file or in `known_profiles`.
+    fn sanitize_profile_inherits(
+        &self,
+        known_profiles: &BTreeSet<String>,
+    ) -> Result<(), ConfigParseErrorKind> {
+        let mut inherit_err_collector = Vec::new();
+
+        self.sanitize_default_profile_inherits(&mut inherit_err_collector);
+        self.sanitize_custom_profile_inherits(&mut inherit_err_collector, known_profiles);
+
+        if !inherit_err_collector.is_empty() {
+            return Err(ConfigParseErrorKind::InheritanceErrors(
+                inherit_err_collector,
+            ));
+        }
+
+        Ok(())
+    }
+
+    /// Check the DefaultProfileImpl and make sure that it doesn't inherit from other
+    /// profiles
+    fn sanitize_default_profile_inherits(&self, inherit_err_collector: &mut Vec<InheritsError>) {
+        if self.default_profile().inherits().is_some() {
+            inherit_err_collector.push(InheritsError::DefaultProfileInheritance(
+                NextestConfig::DEFAULT_PROFILE.to_string(),
+            ));
+        }
+    }
+
+    /// Iterate through each custom profile inherits and report any inheritance error(s).
+    fn sanitize_custom_profile_inherits(
+        &self,
+        inherit_err_collector: &mut Vec<InheritsError>,
+        known_profiles: &BTreeSet<String>,
+    ) {
+        let mut profile_graph = Graph::<&str, (), Directed>::new();
+        let mut profile_map = HashMap::new();
+
+        // Iterate through all custom profiles within the config file and constructs
+        // a reduced graph of the inheritance chain(s)
+        for (name, custom_profile) in self.other_profiles() {
+            let starts_with_default = self.sanitize_custom_default_profile_inherits(
+                name,
+                custom_profile,
+                inherit_err_collector,
+            );
+            if !starts_with_default {
+                // We don't need to add default- profiles. Since they cannot
+                // have inherits specified on them (they effectively always
+                // inherit from default), they cannot participate in inheritance
+                // cycles.
+                self.add_profile_to_graph(
+                    name,
+                    custom_profile,
+                    &mut profile_map,
+                    &mut profile_graph,
+                    inherit_err_collector,
+                    known_profiles,
+                );
+            }
+        }
+
+        self.check_inheritance_cycles(profile_graph, inherit_err_collector);
+    }
+
+    /// Check any CustomProfileImpl that have a "default-" name and make sure they
+    /// do not inherit from other profiles.
+    fn sanitize_custom_default_profile_inherits(
+        &self,
+        name: &str,
+        custom_profile: &CustomProfileImpl,
+        inherit_err_collector: &mut Vec<InheritsError>,
+    ) -> bool {
+        let starts_with_default = name.starts_with("default-");
+
+        if starts_with_default && custom_profile.inherits().is_some() {
+            inherit_err_collector.push(InheritsError::DefaultProfileInheritance(name.to_string()));
+        }
+
+        starts_with_default
+    }
+
+    /// Add the custom profile to the profile graph and collect any inheritance errors like
+    /// self-referential profiles and nonexisting profiles.
+    ///
+    /// `known_profiles` contains profiles from previously loaded (lower priority) files.
+    fn add_profile_to_graph<'cfg>(
+        &self,
+        name: &'cfg str,
+        custom_profile: &'cfg CustomProfileImpl,
+        profile_map: &mut HashMap<&'cfg str, NodeIndex>,
+        profile_graph: &mut Graph<&'cfg str, ()>,
+        inherit_err_collector: &mut Vec<InheritsError>,
+        known_profiles: &BTreeSet<String>,
+    ) {
+        if let Some(inherits_name) = custom_profile.inherits() {
+            if inherits_name == name {
+                inherit_err_collector
+                    .push(InheritsError::SelfReferentialInheritance(name.to_string()))
+            } else if self.get_profile(inherits_name).is_ok() {
+                // Inherited profile exists in this file -- create edge for cycle detection.
+                let from_node = match profile_map.get(name) {
+                    None => {
+                        let profile_node = profile_graph.add_node(name);
+                        profile_map.insert(name, profile_node);
+                        profile_node
+                    }
+                    Some(node_idx) => *node_idx,
+                };
+                let to_node = match profile_map.get(inherits_name) {
+                    None => {
+                        let profile_node = profile_graph.add_node(inherits_name);
+                        profile_map.insert(inherits_name, profile_node);
+                        profile_node
+                    }
+                    Some(node_idx) => *node_idx,
+                };
+                profile_graph.add_edge(from_node, to_node, ());
+            } else if known_profiles.contains(inherits_name) {
+                // Inherited profile exists in a previously loaded file -- valid, no
+                // cycle detection needed (cross-file cycles are impossible with
+                // downward-only inheritance).
+            } else {
+                inherit_err_collector.push(InheritsError::UnknownInheritance(
+                    name.to_string(),
+                    inherits_name.to_string(),
+                ))
+            }
+        }
+    }
+
+    /// Given a profile graph, reports all SCC cycles within the graph using kosaraju algorithm.
+    fn check_inheritance_cycles(
+        &self,
+        profile_graph: Graph<&str, ()>,
+        inherit_err_collector: &mut Vec<InheritsError>,
+    ) {
+        let profile_sccs: Vec<Vec<NodeIndex>> = kosaraju_scc(&profile_graph);
+        let profile_sccs: Vec<Vec<NodeIndex>> = profile_sccs
+            .into_iter()
+            .filter(|scc| scc.len() >= 2)
+            .collect();
+
+        if !profile_sccs.is_empty() {
+            inherit_err_collector.push(InheritsError::InheritanceCycle(
+                profile_sccs
+                    .iter()
+                    .map(|node_idxs| {
+                        let profile_names: Vec<String> = node_idxs
+                            .iter()
+                            .map(|node_idx| profile_graph[*node_idx].to_string())
+                            .collect();
+                        profile_names
+                    })
+                    .collect(),
+            ));
+        }
+    }
 }
 
 // This is the form of `NextestConfig` that gets deserialized.
@@ -1235,6 +1507,8 @@ pub(in crate::config) struct DefaultProfileImpl {
     scripts: Vec<DeserializedProfileScriptConfig>,
     junit: DefaultJunitImpl,
     archive: ArchiveConfig,
+    bench: DefaultBenchConfig,
+    inherits: Inherits,
 }
 
 impl DefaultProfileImpl {
@@ -1279,6 +1553,10 @@ impl DefaultProfileImpl {
             scripts: p.scripts,
             junit: DefaultJunitImpl::for_default_profile(p.junit),
             archive: p.archive.expect("archive present in default profile"),
+            bench: DefaultBenchConfig::for_default_profile(
+                p.bench.expect("bench present in default profile"),
+            ),
+            inherits: Inherits::new(p.inherits),
         }
     }
 
@@ -1286,6 +1564,10 @@ impl DefaultProfileImpl {
         &self.default_filter
     }
 
+    pub(in crate::config) fn inherits(&self) -> Option<&str> {
+        self.inherits.inherits_from()
+    }
+
     pub(in crate::config) fn overrides(&self) -> &[DeserializedOverride] {
         &self.overrides
     }
@@ -1293,6 +1575,20 @@ impl DefaultProfileImpl {
     pub(in crate::config) fn setup_scripts(&self) -> &[DeserializedProfileScriptConfig] {
         &self.scripts
     }
+
+    pub(in crate::config) fn slow_timeout(&self, run_mode: NextestRunMode) -> SlowTimeout {
+        match run_mode {
+            NextestRunMode::Test => self.slow_timeout,
+            NextestRunMode::Benchmark => self.bench.slow_timeout,
+        }
+    }
+
+    pub(in crate::config) fn global_timeout(&self, run_mode: NextestRunMode) -> GlobalTimeout {
+        match run_mode {
+            NextestRunMode::Test => self.global_timeout,
+            NextestRunMode::Benchmark => self.bench.global_timeout,
+        }
+    }
 }
 
 #[derive(Clone, Debug, Deserialize)]
@@ -1337,6 +1633,10 @@ pub(in crate::config) struct CustomProfileImpl {
     junit: JunitImpl,
     #[serde(default)]
     archive: Option<ArchiveConfig>,
+    #[serde(default)]
+    bench: Option<BenchConfig>,
+    #[serde(default)]
+    inherits: Option<String>,
 }
 
 impl CustomProfileImpl {
@@ -1349,6 +1649,27 @@ impl CustomProfileImpl {
         self.default_filter.as_deref()
     }
 
+    pub(in crate::config) fn slow_timeout(&self, run_mode: NextestRunMode) -> Option<SlowTimeout> {
+        match run_mode {
+            NextestRunMode::Test => self.slow_timeout,
+            NextestRunMode::Benchmark => self.bench.as_ref().and_then(|b| b.slow_timeout),
+        }
+    }
+
+    pub(in crate::config) fn global_timeout(
+        &self,
+        run_mode: NextestRunMode,
+    ) -> Option<GlobalTimeout> {
+        match run_mode {
+            NextestRunMode::Test => self.global_timeout,
+            NextestRunMode::Benchmark => self.bench.as_ref().and_then(|b| b.global_timeout),
+        }
+    }
+
+    pub(in crate::config) fn inherits(&self) -> Option<&str> {
+        self.inherits.as_deref()
+    }
+
     pub(in crate::config) fn overrides(&self) -> &[DeserializedOverride] {
         &self.overrides
     }
@@ -1365,6 +1686,10 @@ mod tests {
     use camino_tempfile::tempdir;
     use iddqd::{IdHashItem, IdHashMap, id_hash_map, id_upcast};
 
+    fn tool_name(s: &str) -> ToolName {
+        ToolName::new(s.into()).unwrap()
+    }
+
     /// Test implementation of ConfigWarnings that collects warnings for testing.
     #[derive(Default)]
     struct TestConfigWarnings {
@@ -1379,12 +1704,12 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
             unknown: &BTreeSet<String>,
         ) {
             self.unknown_keys
                 .insert_unique(UnknownKeys {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                     keys: unknown.clone(),
                 })
@@ -1395,12 +1720,12 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
             profiles: &[&str],
         ) {
             self.reserved_profiles
                 .insert_unique(ReservedProfiles {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                     profiles: profiles.iter().map(|&s| s.to_owned()).collect(),
                 })
@@ -1411,13 +1736,13 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
             profile_name: &str,
             empty_count: usize,
         ) {
             self.empty_script_warnings
                 .insert_unique(EmptyScriptSections {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                     profile_name: profile_name.to_owned(),
                     empty_count,
@@ -1429,11 +1754,11 @@ mod tests {
             &mut self,
             config_file: &Utf8Path,
             _workspace_root: &Utf8Path,
-            tool: Option<&str>,
+            tool: Option<&ToolName>,
         ) {
             self.deprecated_scripts
                 .insert_unique(DeprecatedScripts {
-                    tool: tool.map(|s| s.to_owned()),
+                    tool: tool.cloned(),
                     config_file: config_file.to_owned(),
                 })
                 .unwrap();
@@ -1442,58 +1767,58 @@ mod tests {
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct UnknownKeys {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
         keys: BTreeSet<String>,
     }
 
     impl IdHashItem for UnknownKeys {
-        type Key<'a> = Option<&'a str>;
+        type Key<'a> = Option<&'a ToolName>;
         fn key(&self) -> Self::Key<'_> {
-            self.tool.as_deref()
+            self.tool.as_ref()
         }
         id_upcast!();
     }
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct ReservedProfiles {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
         profiles: Vec<String>,
     }
 
     impl IdHashItem for ReservedProfiles {
-        type Key<'a> = Option<&'a str>;
+        type Key<'a> = Option<&'a ToolName>;
         fn key(&self) -> Self::Key<'_> {
-            self.tool.as_deref()
+            self.tool.as_ref()
         }
         id_upcast!();
     }
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct DeprecatedScripts {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
     }
 
     impl IdHashItem for DeprecatedScripts {
-        type Key<'a> = Option<&'a str>;
+        type Key<'a> = Option<&'a ToolName>;
         fn key(&self) -> Self::Key<'_> {
-            self.tool.as_deref()
+            self.tool.as_ref()
         }
         id_upcast!();
     }
 
     #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
     struct EmptyScriptSections {
-        tool: Option<String>,
+        tool: Option<ToolName>,
         config_file: Utf8PathBuf,
         profile_name: String,
         empty_count: usize,
     }
 
     impl IdHashItem for EmptyScriptSections {
-        type Key<'a> = (&'a Option<String>, &'a str);
+        type Key<'a> = (&'a Option<ToolName>, &'a str);
         fn key(&self) -> Self::Key<'_> {
             (&self.tool, &self.profile_name)
         }
@@ -1562,7 +1887,7 @@ mod tests {
             &pcx,
             None,
             &[ToolConfigFile {
-                tool: "my-tool".to_owned(),
+                tool: tool_name("my-tool"),
                 config_file: tool_path.clone(),
             }][..],
             &Default::default(),
@@ -1589,7 +1914,7 @@ mod tests {
                     }
                 },
                 UnknownKeys {
-                    tool: Some("my-tool".to_owned()),
+                    tool: Some(tool_name("my-tool")),
                     config_file: tool_path.clone(),
                     keys: maplit::btreeset! {
                         "store.ignored4".to_owned(),
@@ -1608,7 +1933,7 @@ mod tests {
                     profiles: vec!["default-foo".to_owned()],
                 },
                 ReservedProfiles {
-                    tool: Some("my-tool".to_owned()),
+                    tool: Some(tool_name("my-tool")),
                     config_file: tool_path,
                     profiles: vec!["default-bar".to_owned()],
                 }
@@ -1686,7 +2011,7 @@ mod tests {
             &pcx,
             None,
             &[ToolConfigFile {
-                tool: "tool".to_owned(),
+                tool: tool_name("tool"),
                 config_file: tool_path.clone(),
             }][..],
             &experimental,
@@ -1710,7 +2035,7 @@ mod tests {
                     empty_count: 2,
                 },
                 EmptyScriptSections {
-                    tool: Some("tool".to_owned()),
+                    tool: Some(tool_name("tool")),
                     config_file: tool_path,
                     profile_name: "tool".to_owned(),
                     empty_count: 1,
@@ -1749,7 +2074,7 @@ mod tests {
             &pcx,
             None,
             &[ToolConfigFile {
-                tool: "my-tool".to_owned(),
+                tool: tool_name("my-tool"),
                 config_file: tool_path.clone(),
             }],
             &maplit::btreeset! {ConfigExperimental::SetupScripts},
@@ -1765,7 +2090,7 @@ mod tests {
                     config_file: graph.workspace().root().join(".config/nextest.toml"),
                 },
                 DeprecatedScripts {
-                    tool: Some("my-tool".to_owned()),
+                    tool: Some(tool_name("my-tool")),
                     config_file: tool_path,
                 }
             }
