@@ -1020,7 +1020,7 @@ mod tests {
     use super::*;
     use crate::{
         config::{
-            core::{ConfigExperimental, NextestConfig, ToolConfigFile},
+            core::{ConfigExperimental, NextestConfig, ToolConfigFile, ToolName},
             utils::test_helpers::*,
         },
         errors::{
@@ -1032,8 +1032,13 @@ mod tests {
     use camino_tempfile_ext::prelude::*;
     use indoc::indoc;
     use maplit::btreeset;
+    use nextest_metadata::TestCaseName;
     use test_case::test_case;
 
+    fn tool_name(s: &str) -> ToolName {
+        ToolName::new(s.into()).unwrap()
+    }
+
     #[test]
     fn test_scripts_basic() {
         let config_contents = indoc! {r#"
@@ -1086,7 +1091,7 @@ mod tests {
         let pcx = ParseContext::new(&graph);
 
         let tool_config_files = [ToolConfigFile {
-            tool: "my-tool".to_owned(),
+            tool: tool_name("my-tool"),
             config_file: tool_path.to_path_buf(),
         }];
 
@@ -1126,9 +1131,10 @@ mod tests {
         // This query matches the foo and bar scripts.
         let host_binary_query =
             binary_query(&graph, package_id, "lib", "my-binary", BuildPlatform::Host);
+        let test_name = TestCaseName::new("script1");
         let query = TestQuery {
             binary_query: host_binary_query.to_query(),
-            test_name: "script1",
+            test_name: &test_name,
         };
         let scripts = SetupScripts::new_with_queries(&profile, std::iter::once(query));
         assert_eq!(scripts.len(), 2, "two scripts should be enabled");
@@ -1152,9 +1158,10 @@ mod tests {
         );
 
         // This query matches the baz script.
+        let test_name = TestCaseName::new("script2");
         let query = TestQuery {
             binary_query: target_binary_query.to_query(),
-            test_name: "script2",
+            test_name: &test_name,
         };
         let scripts = SetupScripts::new_with_queries(&profile, std::iter::once(query));
         assert_eq!(scripts.len(), 1, "one script should be enabled");
@@ -1165,9 +1172,10 @@ mod tests {
         );
 
         // This query matches the baz, foo and tool scripts (but note the order).
+        let test_name = TestCaseName::new("script3");
         let query = TestQuery {
             binary_query: target_binary_query.to_query(),
-            test_name: "script3",
+            test_name: &test_name,
         };
         let scripts = SetupScripts::new_with_queries(&profile, std::iter::once(query));
         assert_eq!(scripts.len(), 3, "three scripts should be enabled");
@@ -1526,7 +1534,7 @@ mod tests {
         let tool_path = workspace_dir.child(".config/my-tool.toml");
         tool_path.write_str(tool_config_contents).unwrap();
         let tool_config_files = [ToolConfigFile {
-            tool: "my-tool".to_owned(),
+            tool: tool_name("my-tool"),
             config_file: tool_path.to_path_buf(),
         }];
 
