@@ -6,18 +6,18 @@
 use crate::{
     cargo_config::{TargetTriple, TargetTripleSource},
     config::{
-        core::ConfigExperimental,
+        core::{ConfigExperimental, ToolName},
         elements::{CustomTestGroup, TestGroup},
         scripts::{ProfileScriptType, ScriptId, ScriptType},
     },
     helpers::{display_exited_with, dylib_path_envvar},
+    indenter::{DisplayIndented, indented},
     redact::Redactor,
     reuse_build::{ArchiveFormat, ArchiveStep},
     target_runner::PlatformRunnerSource,
 };
 use camino::{FromPathBufError, Utf8Path, Utf8PathBuf};
 use config::ConfigError;
-use indent_write::{fmt::IndentWriter, indentable::Indented};
 use itertools::{Either, Itertools};
 use nextest_filtering::errors::FiltersetParseErrors;
 use nextest_metadata::RustBinaryId;
@@ -37,12 +37,12 @@ use thiserror::Error;
 #[derive(Debug, Error)]
 #[error(
     "failed to parse nextest config at `{config_file}`{}",
-    provided_by_tool(tool.as_deref())
+    provided_by_tool(tool.as_ref())
 )]
 #[non_exhaustive]
 pub struct ConfigParseError {
     config_file: Utf8PathBuf,
-    tool: Option<String>,
+    tool: Option<ToolName>,
     #[source]
     kind: ConfigParseErrorKind,
 }
@@ -50,12 +50,12 @@ pub struct ConfigParseError {
 impl ConfigParseError {
     pub(crate) fn new(
         config_file: impl Into<Utf8PathBuf>,
-        tool: Option<&str>,
+        tool: Option<&ToolName>,
         kind: ConfigParseErrorKind,
     ) -> Self {
         Self {
             config_file: config_file.into(),
-            tool: tool.map(|s| s.to_owned()),
+            tool: tool.cloned(),
             kind,
         }
     }
@@ -66,8 +66,8 @@ impl ConfigParseError {
     }
 
     /// Returns the tool name associated with this error.
-    pub fn tool(&self) -> Option<&str> {
-        self.tool.as_deref()
+    pub fn tool(&self) -> Option<&ToolName> {
+        self.tool.as_ref()
     }
 
     /// Returns the kind of error this is.
@@ -77,7 +77,7 @@ impl ConfigParseError {
 }
 
 /// Returns the string ` provided by tool <tool>`, if `tool` is `Some`.
-pub fn provided_by_tool(tool: Option<&str>) -> String {
+pub fn provided_by_tool(tool: Option<&ToolName>) -> String {
     match tool {
         Some(tool) => format!(" provided by tool `{tool}`"),
         None => String::new(),
@@ -93,6 +93,9 @@ pub enum ConfigParseErrorKind {
     /// An error occurred while building the config.
     #[error(transparent)]
     BuildError(Box<ConfigError>),
+    /// An error occurred while parsing the config into a table.
+    #[error(transparent)]
+    TomlParseError(Box<toml::de::Error>),
     #[error(transparent)]
     /// An error occurred while deserializing the config.
     DeserializeError(Box<serde_path_to_error::Error<ConfigError>>),
@@ -411,7 +414,7 @@ impl<T: std::error::Error> fmt::Display for ErrorList<T> {
             self.description,
         )?;
         for error in &self.inner {
-            let mut indent = IndentWriter::new_skip_initial("  ", f);
+            let mut indent = indented(f).with_str("  ").skip_initial();
             writeln!(indent, "* {}", DisplayErrorChain::new(error))?;
             f = indent.into_inner();
         }
@@ -461,7 +464,7 @@ where
     E: std::error::Error,
 {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let mut writer = IndentWriter::new(self.initial_indent, f);
+        let mut writer = indented(f).with_str(self.initial_indent);
         write!(writer, "{}", self.error)?;
 
         let Some(mut cause) = self.error.source() else {
@@ -472,7 +475,8 @@ where
 
         loop {
             writeln!(writer)?;
-            let mut indent = IndentWriter::new_skip_initial("    ", writer);
+            // Wrap the existing writer to accumulate indentation.
+            let mut indent = indented(&mut writer).with_str("    ").skip_initial();
             write!(indent, "  - {cause}")?;
 
             let Some(next_cause) = cause.source() else {
@@ -480,7 +484,6 @@ where
             };
 
             cause = next_cause;
-            writer = indent.into_inner();
         }
     }
 }
@@ -597,7 +600,7 @@ impl ProfileScriptErrors {
 
 /// An error which indicates that a profile was requested but not known to nextest.
 #[derive(Clone, Debug, Error)]
-#[error("profile `{profile} not found (known profiles: {})`", .all_profiles.join(", "))]
+#[error("profile `{profile}` not found (known profiles: {})", .all_profiles.join(", "))]
 pub struct ProfileNotFound {
     profile: String,
     all_profiles: Vec<String>,
@@ -641,6 +644,22 @@ pub enum InvalidIdentifier {
     ToolIdentifierInvalidXid(SmolStr),
 }
 
+/// A tool name is invalid.
+#[derive(Clone, Debug, Error, Eq, PartialEq)]
+pub enum InvalidToolName {
+    /// The tool name is empty.
+    #[error("tool name is empty")]
+    Empty,
+
+    /// The tool name is not in the correct Unicode format.
+    #[error("invalid tool name `{0}`")]
+    InvalidXid(SmolStr),
+
+    /// The tool name starts with "@tool", which is reserved for tool identifiers.
+    #[error("tool name cannot start with \"@tool\": `{0}`")]
+    StartsWithToolPrefix(SmolStr),
+}
+
 /// The name of a test group is invalid (not a valid identifier).
 #[derive(Clone, Debug, Error)]
 #[error("invalid custom test group name: {0}")]
@@ -652,7 +671,7 @@ pub struct InvalidCustomTestGroupName(pub InvalidIdentifier);
 pub struct InvalidConfigScriptName(pub InvalidIdentifier);
 
 /// Error returned while parsing a [`ToolConfigFile`](crate::config::core::ToolConfigFile) value.
-#[derive(Clone, Debug, Error)]
+#[derive(Clone, Debug, Error, PartialEq, Eq)]
 pub enum ToolConfigFileParseError {
     #[error(
         "tool-config-file has invalid format: {input}\n(hint: tool configs must be in the format <tool-name>:<path>)"
@@ -663,11 +682,15 @@ pub enum ToolConfigFileParseError {
         input: String,
     },
 
-    /// The tool name was empty.
-    #[error("tool-config-file has empty tool name: {input}")]
-    EmptyToolName {
+    /// The tool name was invalid.
+    #[error("tool-config-file has invalid tool name: {input}")]
+    InvalidToolName {
         /// The input that failed to parse.
         input: String,
+
+        /// The error that occurred.
+        #[source]
+        error: InvalidToolName,
     },
 
     /// The config file path was empty.
@@ -1633,8 +1656,8 @@ pub enum HostPlatformDetectError {
          - `rustc -vV` stderr:\n{}\n\
          - build target error:\n{}\n",
         status,
-        Indented { item: String::from_utf8_lossy(stdout), indent: "  " },
-        Indented { item: String::from_utf8_lossy(stderr), indent: "  " },
+        DisplayIndented { item: String::from_utf8_lossy(stdout), indent: "  " },
+        DisplayIndented { item: String::from_utf8_lossy(stderr), indent: "  " },
         DisplayErrorChain::new_with_initial_indent("  ", build_target_error)
     )]
     RustcVvFailed {
