@@ -10,14 +10,14 @@ use crate::{
         events::*,
         helpers::{Styles, print_lines_in_chunks},
     },
+    run_mode::NextestRunMode,
 };
 use indicatif::{ProgressBar, ProgressDrawTarget, ProgressStyle};
-use nextest_metadata::RustBinaryId;
+use nextest_metadata::{RustBinaryId, TestCaseName};
 use owo_colors::OwoColorize;
 use std::{
     cmp::{max, min},
     env, fmt,
-    io::IsTerminal,
     str::FromStr,
     time::{Duration, Instant},
 };
@@ -112,7 +112,7 @@ pub(super) enum RunningTestStatus {
 #[derive(Debug)]
 pub(super) struct RunningTest {
     binary_id: RustBinaryId,
-    test_name: String,
+    test_name: TestCaseName,
     status: RunningTestStatus,
     start_time: Instant,
     paused_for: Duration,
@@ -158,6 +158,7 @@ impl RunningTest {
 #[derive(Debug)]
 pub(super) struct ProgressBarState {
     bar: ProgressBar,
+    mode: NextestRunMode,
     stats: RunStats,
     running: usize,
     max_progress_running: MaxProgressRunning,
@@ -187,6 +188,7 @@ pub(super) struct ProgressBarState {
 
 impl ProgressBarState {
     pub(super) fn new(
+        mode: NextestRunMode,
         test_count: usize,
         progress_chars: &str,
         max_progress_running: MaxProgressRunning,
@@ -221,6 +223,7 @@ impl ProgressBarState {
 
         Self {
             bar,
+            mode,
             stats: RunStats::default(),
             running: 0,
             max_progress_running,
@@ -292,7 +295,7 @@ impl ProgressBarState {
                 msg.push_str(&format!(
                     "\n             ... and {} more {} running",
                     overflow_count.style(styles.count),
-                    plural::tests_str(overflow_count),
+                    plural::tests_str(self.mode, overflow_count),
                 ));
                 count += 1;
             }
@@ -346,8 +349,8 @@ impl ProgressBarState {
 
                 if let Some(running_tests) = &mut self.running_tests {
                     running_tests.push(RunningTest {
-                        binary_id: test_instance.id().binary_id.clone(),
-                        test_name: test_instance.id().test_name.to_owned(),
+                        binary_id: test_instance.binary_id.clone(),
+                        test_name: test_instance.test_name.to_owned(),
                         status: RunningTestStatus::Running,
                         start_time: Instant::now(),
                         paused_for: Duration::ZERO,
@@ -361,7 +364,7 @@ impl ProgressBarState {
                 ..
             } => {
                 self.running = *running;
-                self.remove_test(&test_instance.id());
+                self.remove_test(test_instance);
 
                 self.bar.set_prefix(progress_bar_prefix(
                     current_stats,
@@ -378,11 +381,11 @@ impl ProgressBarState {
                 delay_before_next_attempt,
                 ..
             } => {
-                self.remove_test(&test_instance.id());
+                self.remove_test(test_instance);
                 if let Some(running_tests) = &mut self.running_tests {
                     running_tests.push(RunningTest {
-                        binary_id: test_instance.id().binary_id.clone(),
-                        test_name: test_instance.id().test_name.to_owned(),
+                        binary_id: test_instance.binary_id.clone(),
+                        test_name: test_instance.test_name.to_owned(),
                         status: RunningTestStatus::Delay(*delay_before_next_attempt),
                         start_time: Instant::now(),
                         paused_for: Duration::ZERO,
@@ -390,11 +393,11 @@ impl ProgressBarState {
                 }
             }
             TestEventKind::TestRetryStarted { test_instance, .. } => {
-                self.remove_test(&test_instance.id());
+                self.remove_test(test_instance);
                 if let Some(running_tests) = &mut self.running_tests {
                     running_tests.push(RunningTest {
-                        binary_id: test_instance.id().binary_id.clone(),
-                        test_name: test_instance.id().test_name.to_owned(),
+                        binary_id: test_instance.binary_id.clone(),
+                        test_name: test_instance.test_name.to_owned(),
                         status: RunningTestStatus::Retry,
                         start_time: Instant::now(),
                         paused_for: Duration::ZERO,
@@ -406,8 +409,8 @@ impl ProgressBarState {
                     running_tests
                         .iter_mut()
                         .find(|rt| {
-                            &rt.binary_id == test_instance.id().binary_id
-                                && rt.test_name == test_instance.id().test_name
+                            &rt.binary_id == test_instance.binary_id
+                                && &rt.test_name == test_instance.test_name
                         })
                         .expect("a slow test to be already running")
                         .status = RunningTestStatus::Slow;
@@ -467,7 +470,7 @@ impl ProgressBarState {
                     .iter()
                     .position(|e| {
                         &e.binary_id == test_instance.binary_id
-                            && e.test_name == test_instance.test_name
+                            && &e.test_name == test_instance.test_name
                     })
                     .expect("finished test to have started"),
             );
@@ -510,7 +513,7 @@ pub(super) struct TerminalProgress {
 impl TerminalProgress {
     const ENV: &str = "CARGO_TERM_PROGRESS_TERM_INTEGRATION";
 
-    pub(super) fn new(configs: &CargoConfigs, stream: &dyn IsTerminal) -> Option<Self> {
+    pub(super) fn new(configs: &CargoConfigs, is_terminal: bool) -> Option<Self> {
         // See whether terminal integration is enabled in Cargo.
         for config in configs.discovered_configs() {
             match config {
@@ -561,7 +564,7 @@ impl TerminalProgress {
             }
         }
 
-        supports_osc_9_4(stream).then(Self::default)
+        supports_osc_9_4(is_terminal).then(Self::default)
     }
 
     pub(super) fn update_progress(&mut self, event: &TestEvent<'_>) {
@@ -617,8 +620,8 @@ impl TerminalProgress {
 }
 
 /// Determines whether the terminal supports ANSI OSC 9;4.
-fn supports_osc_9_4(stream: &dyn IsTerminal) -> bool {
-    if !stream.is_terminal() {
+fn supports_osc_9_4(is_terminal: bool) -> bool {
+    if !is_terminal {
         debug!(
             "autodetect terminal progress reporting: disabling since \
              passed-in stream (usually stderr) is not a terminal"
@@ -720,10 +723,11 @@ pub(super) fn write_summary_str(run_stats: &RunStats, styles: &Styles, out: &mut
         setup_scripts_timed_out: _,
         passed,
         passed_slow,
+        passed_timed_out: _,
         flaky,
         failed,
         failed_slow: _,
-        timed_out,
+        failed_timed_out,
         leaky,
         leaky_failed,
         exec_failed,
@@ -792,11 +796,11 @@ pub(super) fn write_summary_str(run_stats: &RunStats, styles: &Styles, out: &mut
         );
     }
 
-    if timed_out > 0 {
+    if failed_timed_out > 0 {
         swrite!(
             out,
             "{} {}, ",
-            timed_out.style(styles.count),
+            failed_timed_out.style(styles.count),
             "timed out".style(styles.fail),
         );
     }
@@ -946,7 +950,7 @@ mod tests {
 
     fn running_test_examples(now: Instant) -> Vec<(&'static str, RunningTest)> {
         let binary_id = RustBinaryId::new("my-binary");
-        let test_name = "test::my_test".to_string();
+        let test_name = TestCaseName::new("test::my_test");
         let start_time = now - Duration::from_secs(125); // 2 minutes 5 seconds ago
 
         vec![
@@ -1051,7 +1055,7 @@ mod tests {
                 RunStats {
                     initial_run_count: 20,
                     finished_count: 10,
-                    timed_out: 1,
+                    failed_timed_out: 1,
                     cancel_reason: Some(CancelReason::TestFailure),
                     ..RunStats::default()
                 },
