@@ -30,6 +30,7 @@ use crate::{
 use chrono::Local;
 use debug_ignore::DebugIgnore;
 use futures::future::{Fuse, FusedFuture};
+use nextest_metadata::MismatchReason;
 use quick_junit::ReportUuid;
 use std::{collections::BTreeMap, env, pin::Pin, time::Duration};
 use tokio::{
@@ -656,6 +657,7 @@ where
             InternalEvent::Executor(ExecutorEvent::Started {
                 stress_index,
                 test_instance,
+                command_line,
                 req_rx_tx,
             }) => {
                 if self.run_stats.cancel_reason.is_some() {
@@ -675,9 +677,10 @@ where
                 self.new_test(test_instance, req_tx);
                 self.callback_none_response(TestEventKind::TestStarted {
                     stress_index,
-                    test_instance,
+                    test_instance: test_instance.id(),
                     current_stats: self.run_stats,
                     running: self.running_tests.len(),
+                    command_line,
                 })
             }
             InternalEvent::Executor(ExecutorEvent::Slow {
@@ -707,7 +710,7 @@ where
 
                 self.callback_none_response(TestEventKind::TestSlow {
                     stress_index,
-                    test_instance,
+                    test_instance: test_instance.id(),
                     retry_data,
                     elapsed,
                     will_terminate: will_terminate.is_some(),
@@ -724,7 +727,7 @@ where
                 instance.attempt_failed_will_retry(run_status.clone());
                 self.callback_none_response(TestEventKind::TestAttemptFailedWillRetry {
                     stress_index,
-                    test_instance,
+                    test_instance: test_instance.id(),
                     failure_output,
                     run_status,
                     delay_before_next_attempt,
@@ -735,6 +738,7 @@ where
                 stress_index,
                 test_instance,
                 retry_data,
+                command_line,
                 tx,
             }) => {
                 if self.run_stats.cancel_reason.is_some() {
@@ -754,9 +758,10 @@ where
 
                 self.callback_none_response(TestEventKind::TestRetryStarted {
                     stress_index,
-                    test_instance,
+                    test_instance: test_instance.id(),
                     retry_data,
                     running: self.running_tests.len(),
+                    command_line,
                 })
             }
             InternalEvent::Executor(ExecutorEvent::Finished {
@@ -777,7 +782,7 @@ where
 
                 self.basic_callback(TestEventKind::TestFinished {
                     stress_index,
-                    test_instance,
+                    test_instance: test_instance.id(),
                     success_output,
                     failure_output,
                     junit_store_success_output,
@@ -818,10 +823,15 @@ where
                 test_instance,
                 reason,
             }) => {
-                self.run_stats.skipped += 1;
+                // If the mismatch reason is that this test isn't a benchmark,
+                // we don't display it in the skip counts (but still keep track
+                // of it internally).
+                if !matches!(reason, MismatchReason::NotBenchmark) {
+                    self.run_stats.skipped += 1;
+                }
                 self.callback_none_response(TestEventKind::TestSkipped {
                     stress_index,
-                    test_instance,
+                    test_instance: test_instance.id(),
                     reason,
                 })
             }
