@@ -10,8 +10,8 @@ use crate::{
     },
     double_spawn::DoubleSpawnInfo,
     errors::{
-        ConfigureHandleInheritanceError, StressCountParseError, TestRunnerBuildError,
-        TestRunnerExecuteErrors,
+        ConfigureHandleInheritanceError, DebuggerCommandParseError, StressCountParseError,
+        TestRunnerBuildError, TestRunnerExecuteErrors, TracerCommandParseError,
     },
     input::{InputHandler, InputHandlerKind, InputHandlerStatus},
     list::{TestInstanceWithSettings, TestList},
@@ -36,6 +36,186 @@ use tokio::{
 };
 use tracing::{debug, warn};
 
+/// A parsed debugger command.
+#[derive(Clone, Debug)]
+pub struct DebuggerCommand {
+    program: String,
+    args: Vec<String>,
+}
+
+impl DebuggerCommand {
+    /// Gets the program.
+    pub fn program(&self) -> &str {
+        // The from_str constructor ensures that there is at least one part.
+        &self.program
+    }
+
+    /// Gets the arguments.
+    pub fn args(&self) -> &[String] {
+        &self.args
+    }
+}
+
+impl FromStr for DebuggerCommand {
+    type Err = DebuggerCommandParseError;
+
+    fn from_str(command: &str) -> Result<Self, Self::Err> {
+        let mut parts =
+            shell_words::split(command).map_err(DebuggerCommandParseError::ShellWordsParse)?;
+        if parts.is_empty() {
+            return Err(DebuggerCommandParseError::EmptyCommand);
+        }
+        let program = parts.remove(0);
+        Ok(Self {
+            program,
+            args: parts,
+        })
+    }
+}
+
+/// A parsed tracer command.
+#[derive(Clone, Debug)]
+pub struct TracerCommand {
+    program: String,
+    args: Vec<String>,
+}
+
+impl TracerCommand {
+    /// Gets the program.
+    pub fn program(&self) -> &str {
+        &self.program
+    }
+
+    /// Gets the arguments.
+    pub fn args(&self) -> &[String] {
+        &self.args
+    }
+}
+
+impl FromStr for TracerCommand {
+    type Err = TracerCommandParseError;
+
+    fn from_str(command: &str) -> Result<Self, Self::Err> {
+        let mut parts =
+            shell_words::split(command).map_err(TracerCommandParseError::ShellWordsParse)?;
+        if parts.is_empty() {
+            return Err(TracerCommandParseError::EmptyCommand);
+        }
+        let program = parts.remove(0);
+        Ok(Self {
+            program,
+            args: parts,
+        })
+    }
+}
+
+/// An interceptor wraps test execution with a debugger or tracer.
+#[derive(Clone, Debug, Default)]
+pub enum Interceptor {
+    /// No interceptor - standard test execution.
+    #[default]
+    None,
+
+    /// Run the test under a debugger.
+    Debugger(DebuggerCommand),
+
+    /// Run the test under a syscall tracer.
+    Tracer(TracerCommand),
+}
+
+impl Interceptor {
+    /// Returns true if timeouts should be disabled.
+    ///
+    /// Both debuggers and tracers disable timeouts.
+    pub fn should_disable_timeouts(&self) -> bool {
+        match self {
+            Interceptor::None => false,
+            Interceptor::Debugger(_) | Interceptor::Tracer(_) => true,
+        }
+    }
+
+    /// Returns true if stdin should be passed through to child test processes.
+    ///
+    /// Only debuggers need stdin passthrough for interactive debugging.
+    pub fn should_passthrough_stdin(&self) -> bool {
+        match self {
+            Interceptor::None | Interceptor::Tracer(_) => false,
+            Interceptor::Debugger(_) => true,
+        }
+    }
+
+    /// Returns true if a process group should be created for the child.
+    ///
+    /// Debuggers need terminal control, so no process group is created. Tracers
+    /// work fine with process groups.
+    pub fn should_create_process_group(&self) -> bool {
+        match self {
+            Interceptor::None | Interceptor::Tracer(_) => true,
+            Interceptor::Debugger(_) => false,
+        }
+    }
+
+    /// Returns true if leak detection should be skipped.
+    ///
+    /// Both debuggers and tracers skip leak detection to avoid interference.
+    pub fn should_skip_leak_detection(&self) -> bool {
+        match self {
+            Interceptor::None => false,
+            Interceptor::Debugger(_) | Interceptor::Tracer(_) => true,
+        }
+    }
+
+    /// Returns true if the test command should be displayed.
+    ///
+    /// Used to determine if we should print the wrapper command for debugging.
+    pub fn should_show_wrapper_command(&self) -> bool {
+        match self {
+            Interceptor::None => false,
+            Interceptor::Debugger(_) | Interceptor::Tracer(_) => true,
+        }
+    }
+
+    /// Returns true if, on receiving SIGTSTP, we should send SIGTSTP to the
+    /// child.
+    ///
+    /// Debugger mode has special signal handling where we don't send SIGTSTP to
+    /// the child (it receives it directly from the terminal, since no process
+    /// group is created). Tracers use standard signal handling.
+    pub fn should_send_sigtstp(&self) -> bool {
+        match self {
+            Interceptor::None | Interceptor::Tracer(_) => true,
+            Interceptor::Debugger(_) => false,
+        }
+    }
+}
+
+/// A child process identifier: either a single process or a process group.
+#[derive(Copy, Clone, Debug)]
+pub(super) enum ChildPid {
+    /// A single process ID.
+    Process(#[cfg_attr(not(unix), expect(unused))] u32),
+
+    /// A process group ID.
+    #[cfg(unix)]
+    ProcessGroup(u32),
+}
+
+impl ChildPid {
+    /// Returns the PID value to use with `libc::kill`.
+    ///
+    /// - `Process(pid)` returns `pid as i32` (positive, kills single process).
+    /// - `ProcessGroup(pid)` returns `-(pid as i32)` (negative, kills process group).
+    ///
+    /// On Windows, always returns `pid as i32`.
+    #[cfg(unix)]
+    pub(super) fn for_kill(self) -> i32 {
+        match self {
+            ChildPid::Process(pid) => pid as i32,
+            ChildPid::ProcessGroup(pid) => -(pid as i32),
+        }
+    }
+}
+
 /// Test runner options.
 #[derive(Debug, Default)]
 pub struct TestRunnerBuilder {
@@ -44,6 +224,7 @@ pub struct TestRunnerBuilder {
     max_fail: Option<MaxFail>,
     test_threads: Option<TestThreads>,
     stress_condition: Option<StressCondition>,
+    interceptor: Interceptor,
 }
 
 impl TestRunnerBuilder {
@@ -86,6 +267,12 @@ impl TestRunnerBuilder {
         self
     }
 
+    /// Sets the interceptor (debugger or tracer) to use for running tests.
+    pub fn set_interceptor(&mut self, interceptor: Interceptor) -> &mut Self {
+        self.interceptor = interceptor;
+        self
+    }
+
     /// Creates a new test runner.
     #[expect(clippy::too_many_arguments)]
     pub fn build<'a>(
@@ -132,6 +319,7 @@ impl TestRunnerBuilder {
                 cli_args,
                 max_fail,
                 stress_condition: self.stress_condition,
+                interceptor: self.interceptor,
                 runtime,
             },
             signal_handler,
@@ -283,6 +471,7 @@ struct TestRunnerInner<'a> {
     cli_args: Vec<String>,
     max_fail: MaxFail,
     stress_condition: Option<StressCondition>,
+    interceptor: Interceptor,
     runtime: Runtime,
 }
 
@@ -299,6 +488,13 @@ impl<'a> TestRunnerInner<'a> {
     {
         // TODO: add support for other test-running approaches, measure performance.
 
+        // Disable the global timeout when an interceptor is active.
+        let global_timeout = if self.interceptor.should_disable_timeouts() {
+            crate::time::far_future_duration()
+        } else {
+            self.profile.global_timeout(self.test_list.mode()).period
+        };
+
         let mut dispatcher_cx = DispatcherContext::new(
             callback,
             self.run_id,
@@ -306,7 +502,7 @@ impl<'a> TestRunnerInner<'a> {
             self.cli_args.clone(),
             self.test_list.run_count(),
             self.max_fail,
-            self.profile.global_timeout().period,
+            global_timeout,
             self.stress_condition.clone(),
         );
 
@@ -318,6 +514,7 @@ impl<'a> TestRunnerInner<'a> {
             self.target_runner.clone(),
             self.capture_strategy,
             self.force_retries,
+            self.interceptor.clone(),
         );
 
         // Send the initial event.
@@ -614,4 +811,29 @@ mod tests {
         assert_eq!(runner.inner.capture_strategy, CaptureStrategy::None);
         assert_eq!(runner.inner.test_threads, 1, "tests run serially");
     }
+
+    #[test]
+    fn test_debugger_command_parsing() {
+        // Valid commands
+        let cmd = DebuggerCommand::from_str("gdb --args").unwrap();
+        assert_eq!(cmd.program(), "gdb");
+        assert_eq!(cmd.args(), &["--args"]);
+
+        let cmd = DebuggerCommand::from_str("rust-gdb -ex run --args").unwrap();
+        assert_eq!(cmd.program(), "rust-gdb");
+        assert_eq!(cmd.args(), &["-ex", "run", "--args"]);
+
+        // With quotes
+        let cmd = DebuggerCommand::from_str(r#"gdb -ex "set print pretty on" --args"#).unwrap();
+        assert_eq!(cmd.program(), "gdb");
+        assert_eq!(cmd.args(), &["-ex", "set print pretty on", "--args"]);
+
+        // Empty command
+        let err = DebuggerCommand::from_str("").unwrap_err();
+        assert!(matches!(err, DebuggerCommandParseError::EmptyCommand));
+
+        // Whitespace only
+        let err = DebuggerCommand::from_str("   ").unwrap_err();
+        assert!(matches!(err, DebuggerCommandParseError::EmptyCommand));
+    }
 }
