@@ -1,9 +1,8 @@
 // Copyright (c) The nextest Contributors
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-//! Basic tests for the test runner.
+//! Integration tests for nextest-runner.
 
-mod basic;
 mod fixtures;
 mod target_runner;
 mod target_triple;
